import math
import numpy as np
from janus.utils.ClimateUtilities import * #To get the math methods routines
import janus.utils.water_tables as wt
#
#All units are mks units
#
#ToDo:
#       * Add conversion factors (calories to joule,Megatons, etc.)
#       * Unit conversion calculating object
#       * Database of interesting constants (e.g energy and C
#          content of coal)
#       * Find a better way to organize database of contants,
#         allowing indexing,unit information and description
#       *Add the rest of the gases to the database, and find a
#        better way to index them and provide help. Note that
#        a printed table is more "transparent" in terms of what
#        data is available. Find a way of providing a similar
#        tabular summary of available data, and perhaps values
#        This applies to the planet data table as well, and perhaps
#        even to the table of physical constants.
#
#       *Finish putting data into the gas database,
#        perhaps including Van der Waals, Shomate and Antoine
#        coefficients, at least in selected cases
#
#

#-------------Basic physical constants-------------------
#
#The following five are the most accurate 1986 values 
#
h = 6.626075540e-34    #Planck's constant
c = 2.99792458e8       #Speed of light
k =1.38065812e-23      #Boltzman thermodynamic constant
sigma = 5.67051196e-8  #Stefan-Boltzman constant
G = 6.67428e-11        #Gravitational constant (2006 measurements)
#

#-----------Thermodynamic constants------------
#Following will come out in J/(deg kmol), so
#that dividing Rstar by molecular weight gives
#gas constant appropriate for mks units
N_avogadro = 6.022136736e23  #Avogadro's number
Rstar = 1000.*k*N_avogadro   #Universal gas constant
#


#----------Properties of gases-----------------
#The following are approximate mean values
#for "normal" temperatures and pressures, suitable only
#for rough calculations.
#


# Molar masses of each species (note the units)
molar_mass = {
        "H2O" : 0.01801528,     # kg mol−1
        "CO2" : 0.04401,        # kg mol−1
        "H2"  : 0.00201588,     # kg mol−1
        "CH4" : 0.01604,        # kg mol−1
        "CO"  : 0.02801,        # kg mol−1
        "N2"  : 0.028014,       # kg mol−1
        "O2"  : 0.031999,       # kg mol−1
        "SO2" : 0.064066,       # kg mol−1
        "H2S" : 0.0341,         # kg mol−1 
        "H"   : 0.001008,       # kg mol−1 
        "C"   : 0.012011,       # kg mol−1 
        "O"   : 0.015999,       # kg mol−1 
        "N"   : 0.014007,       # kg mol−1 
        "S"   : 0.03206,        # kg mol−1 
        "He"  : 0.0040026,      # kg mol−1 
        "NH3" : 0.017031,       # kg mol−1 
        "O3"  : 0.0479982,      # kg mol-1
        "N2O" : 0.0440128, 
        "NO"  : 0.0300061, 
        "NO2" : 0.0460055, 
        "HNO3": 0.06301284, 
        "OCS" : 0.060075,       
        "S2"  : 0.0641,               # kg mol-1
      
    }

#This class allows convenient access
#to the basic thermodynamic properties of
#a gas, and selected properties of its
#solid and liquid phases. The properties
#do not need to be specified in the __init__
#method, since they can be created dynamically.
#We specify them anyway, and set them to None,
#as a guide to the naming conventions for those
#creating their own gas objects. A utility is also
#available which will turn a LaTeX formatted thermodynamic table
#into a Python script defining new gas objects.
#
#
#
#ToDo: *Add more documentation and help features
#       
#      *provide dictionary of properties and units.
#      *Add some methods or lists that make it easier
#       for the user to create new gas objects and insert
#       the data
class gas:
    '''
    A gas object stores thermodynamic data for
    a gas, and selected properties of its condensed
    phases.  You can create a gas object for gas
    G by executing:
             G = gas()
    and then setting the attributes individually (see
    below for explanation of names).  A collection
    of gas objects for common gases is provided as part
    of the phys module.  These gas objects were not actually
    created "by hand" but rather using a utility script that
    automatically translates a LaTeX formatted table into a
    Python script defining the objects.


    Attributes of a gas object:
        CriticalPointT:  Critical point temperature (K)
        CriticalPointP:  Critical point pressure (Pa)
        TriplePointT: Triple point temperature (K)
        TriplePointP: Triple point pressure (Pa)
        L_vaporization_BoilingPoint: Latent heat of vaporization (J/kg)
            at boiling point.  The so-called "boiling point" is
            the temperature at which the saturation vapor pressure
            equals 1 atmosphere (1.013 bar). For CO2, the "boiling point"
            occurs below the triple point temperature, so the condensed
            phase would not be a liquid. Hence, for CO2 the
            latent heat is given at the arbitrary reference point
            of 253K and 29Pa.
        L_vaporization_TriplePoint: Latent heat of vaporization (J/kg)
            at the triple point
        L_fusion: Latent heat of fusion (J/kg) at the triple point
        L_sublimation: Latent heat of sublimation (J/kg) at triple point
        rho_liquid_BoilingPoint Liquid phase density (kg/m**3)
            at the boiling point
        rho_liquid_TriplePoint: Liquid phase density (kg/m**3)
            at the triple point
        rho_solid: Solid phase density (kg/m**3) at (or sometimes near)
            the triple point
        cp: Gas phase specific heat (J/(kg K)), at 298K and 1 bar
        gamma: ratio of specific heat at constant pressure
            to specific heat at constant volume. (Generally
            stated at 298K and 1bar)
        MolecularWeight: Molecular weight of the dominant isotope
        name: Name of the gas 
        formula: Chemical formula (e.g. 'CH4')

        L_vaporization: Default value to use for latent heat of
            vaporization.  Set to triple point value, if available,
            else to boiling point value
        rho_liquid: Default value to use for liquid phase density.
            Set to triple point value if available otherwise
            set to boiling point value

        R: Gas constant for the individual gas. Computed from
            other data as Rstar/MolecularWeight, when the update()
            method is called
        Rcp: The adiabatic exponent R/cp. Computed from other
            data when the update() method is called.
        
    '''

    #The __repr__ method allows us to print out
    #a help string when the user types the name
    #of a gas object
    def __repr__(self):
        firstline =\
        'This gas object contains thermodynamic data on %s\n'%self.formula
        secondline = \
        'Type \"help(gas)\" for more information\n'
        return firstline+secondline
    def __init__(self):
        self.CriticalPointT = None
        self.CriticalPointP = None
        self.TriplePointT = None
        self.TriplePointP = None
        self.L_vaporization_BoilingPoint = None
        self.L_vaporization_TriplePoint = None
        self.L_fusion = None
        self.L_sublimation = None
        self.rho_liquid_BoilingPoint = None
        self.rho_liquid_TriplePoint = None
        self.rho_solid = None
        self.cp = None
        self.gamma = None
        self.MolecularWeight = None
        self.name = None 
        self.formula = None
        #
        #Default values for latent heat and liquid
        #density.  The triple point value is set
        #as the default if it is available, otherwise
        #the boiling point values are used.
        self.L_vaporization= None
        self.rho_liquid= None
        #
        #Computed quantities
        #
        self.R = None
        self.Rcp = None
    #Function to compute derived properties
    def update(self):
        self.R = Rstar/self.MolecularWeight
        self.Rcp = self.R/self.cp

#Set properties of individual gases
#
#Thermodynamic properties of dry Earth air
air = gas()
air.name = 'Earth Air'
air.cp = 1004.
air.MolecularWeight = 28.97
air.gamma = 1.4003

R_gas = 8.31446261815324 # Universal gas constant, J.K-1.mol-1

#------------------------
H2O = gas()
H2O.CriticalPointT = 6.471000e+02
H2O.CriticalPointP = 2.210000e+07
H2O.TriplePointT = 2.731500e+02
H2O.TriplePointP = 6.110000e+02
H2O.L_vaporization_BoilingPoint = 2.255000e+06
H2O.L_vaporization_TriplePoint = 2.493000e+06
H2O.L_fusion = 3.340000e+05
H2O.L_sublimation = 2.840000e+06
H2O.rho_liquid_BoilingPoint = 9.584000e+02
H2O.rho_liquid_TriplePoint = 9.998700e+02
H2O.rho_solid = 9.170000e+02
H2O.cp = 1.847000e+03
H2O.gamma = 1.331000e+00
H2O.MolecularWeight = 1.800000e+01
H2O.name = 'Water' 
H2O.formula = 'H2O' 
H2O.L_vaporization=2.493000e+06
H2O.rho_liquid=9.998700e+02
#------------------------
CH4 = gas()
CH4.CriticalPointT = 1.904400e+02
CH4.CriticalPointP = 4.596000e+06
CH4.TriplePointT = 9.067000e+01
CH4.TriplePointP = 1.170000e+04
CH4.L_vaporization_BoilingPoint = 5.100000e+05
CH4.L_vaporization_TriplePoint = 5.360000e+05
CH4.L_fusion = 5.868000e+04
CH4.L_sublimation = 5.950000e+05
CH4.rho_liquid_BoilingPoint = 4.502000e+02
CH4.rho_liquid_TriplePoint = None
CH4.rho_solid = 5.093000e+02
CH4.cp = 2.195000e+03
CH4.gamma = 1.305000e+00
CH4.MolecularWeight = 1.600000e+01
CH4.name = 'Methane' 
CH4.formula = 'CH4' 
CH4.L_vaporization=5.360000e+05
CH4.rho_liquid=4.502000e+02
#------------------------
CO2 = gas()
CO2.CriticalPointT = 3.042000e+02
CO2.CriticalPointP = 7.382500e+06
CO2.TriplePointT = 2.165400e+02
CO2.TriplePointP = 5.185000e+05
CO2.L_vaporization_BoilingPoint = None
CO2.L_vaporization_TriplePoint = 3.970000e+05
CO2.L_fusion = 1.960000e+05
CO2.L_sublimation = 5.930000e+05
CO2.rho_liquid_BoilingPoint = 1.032000e+03
CO2.rho_liquid_TriplePoint = 1.110000e+03
CO2.rho_solid = 1.562000e+03
CO2.cp = 8.200000e+02
CO2.gamma = 1.294000e+00
CO2.MolecularWeight = 4.400000e+01
CO2.name = 'Carbon Dioxide' 
CO2.formula = 'CO2' 
CO2.L_vaporization=3.970000e+05
CO2.rho_liquid=1.110000e+03
#------------------------
CO = gas() # Complete Thermodynamic data of CO (ToDo: confirm the "None")
CO.CriticalPointT = 1.134450e+02
CO.CriticalPointP = 3.498750e+06
CO.TriplePointT = 6.795000e+01
CO.TriplePointP = 1.530000e+04
CO.L_vaporization_BoilingPoint = None
CO.L_vaporization_TriplePoint = 2.142857e+05 # 
CO.L_fusion = None
CO.L_sublimation = 2.7142857e+05
CO.rho_liquid_BoilingPoint = None
CO.rho_liquid_TriplePoint = None
CO.rho_solid = None
CO.cp = 1.04000e+03
CO.gamma = 50.241546e+00
CO.MolecularWeight = 2.800970e+01
CO.name = 'Carbon Monoxide' 
CO.formula = 'CO' 
CO.L_vaporization=2.142857e+05
CO.rho_liquid=None
#------------------------
N2 = gas()
N2.CriticalPointT = 1.262000e+02
N2.CriticalPointP = 3.400000e+06
N2.TriplePointT = 6.314000e+01
N2.TriplePointP = 1.253000e+04
N2.L_vaporization_BoilingPoint = 1.980000e+05
N2.L_vaporization_TriplePoint = 2.180000e+05
N2.L_fusion = 2.573000e+04
N2.L_sublimation = 2.437000e+05
N2.rho_liquid_BoilingPoint = 8.086000e+02
N2.rho_liquid_TriplePoint = None
N2.rho_solid = 1.026000e+03
N2.cp = 1.037000e+03
N2.gamma = 1.403000e+00
N2.MolecularWeight = 2.801340e+01
N2.name = 'Nitrogen' 
N2.formula = 'N2' 
N2.L_vaporization=2.180000e+05
N2.rho_liquid=8.086000e+02
#------------------------
O2 = gas()
O2.CriticalPointT = 1.545400e+02
O2.CriticalPointP = 5.043000e+06
O2.TriplePointT = 5.430000e+01
O2.TriplePointP = 1.500000e+02
O2.L_vaporization_BoilingPoint = 2.130000e+05
O2.L_vaporization_TriplePoint = 2.420000e+05
O2.L_fusion = 1.390000e+04
O2.L_sublimation = 2.560000e+05
O2.rho_liquid_BoilingPoint = 1.141000e+03
O2.rho_liquid_TriplePoint = 1.307000e+03
O2.rho_solid = 1.351000e+03
O2.cp = 9.160000e+02
O2.gamma = 1.393000e+00
O2.MolecularWeight = 3.200000e+01
O2.name = 'Oxygen' 
O2.formula = 'O2' 
O2.L_vaporization=2.420000e+05
O2.rho_liquid=1.307000e+03
#------------------------
O3 = gas() # https://encyclopedia.airliquide.com/ozone#properties
O3.CriticalPointT = 2.61e+02  
O3.CriticalPointP = 5.57e6
O3.TriplePointT = 8.015e+01  
O3.TriplePointP = 7.346e-01
O3.L_vaporization_BoilingPoint = 2.8849e+05   
O3.L_vaporization_TriplePoint = None # left to find
O3.L_fusion = 4.1668e+04
O3.L_sublimation = 0.1e-99 # !CAREFUL! Actually: None
O3.rho_liquid_BoilingPoint = 1.34908e+03
O3.rho_liquid_TriplePoint = None # left to find
O3.rho_solid = 1.351000e+03 # left to find
O3.cp = 8.197165e+02 # https://webbook.nist.gov/cgi/cbook.cgi?ID=C10028156&Mask=1&Type=JANAFG&Table=on#JANAFG
O3.gamma = 1.393000e+00  # left to find cv
O3.MolecularWeight = 4.798e+01
O3.name = 'Ozone' 
O3.formula = 'O3' 
O3.L_vaporization=2.8849e+05 # left to find L_vaporization_TriplePoint
O3.rho_liquid=1.34908e+03
#------------------------
H2 = gas()
H2.CriticalPointT = 3.320000e+01
H2.CriticalPointP = 1.298000e+06
H2.TriplePointT = 1.395000e+01
H2.TriplePointP = 7.200000e+03
H2.L_vaporization_BoilingPoint = 4.540000e+05
H2.L_vaporization_TriplePoint = None
H2.L_fusion = 5.820000e+04
H2.L_sublimation = 0.1e-99 # !CAREFUL! Actually: None
H2.rho_liquid_BoilingPoint = 7.097000e+01
H2.rho_liquid_TriplePoint = None
H2.rho_solid = 8.800000e+01
H2.cp = 1.423000e+04
H2.gamma = 1.384000e+00
H2.MolecularWeight = 2.015680e+00 # original: 2.000000e+00
H2.name = 'Hydrogen' 
H2.formula = 'H2' 
H2.L_vaporization=4.540000e+05
H2.rho_liquid=7.097000e+01
#------------------------
He = gas()
He.CriticalPointT = 5.100000e+00
He.CriticalPointP = 2.280000e+05
He.TriplePointT = 2.170000e+00
He.TriplePointP = 5.070000e+03
He.L_vaporization_BoilingPoint = 2.030000e+04
He.L_vaporization_TriplePoint = None
He.L_fusion = None
He.L_sublimation = 0.1e-99 # !CAREFUL! Actually: None
He.rho_liquid_BoilingPoint = 1.249600e+02
He.rho_liquid_TriplePoint = None
He.rho_solid = 2.000000e+02
He.cp = 5.196000e+03
He.gamma = 1.664000e+00
He.MolecularWeight = 4.002602e+00
He.name = 'Helium' 
He.formula = 'He' 
He.L_vaporization=2.030000e+04
He.rho_liquid=1.249600e+02
#------------------------
NH3 = gas()
NH3.CriticalPointT = 4.055000e+02
NH3.CriticalPointP = 1.128000e+07
NH3.TriplePointT = 1.954000e+02
NH3.TriplePointP = 6.100000e+03
NH3.L_vaporization_BoilingPoint = 1.371000e+06
NH3.L_vaporization_TriplePoint = 1.658000e+06
NH3.L_fusion = 3.314000e+05
NH3.L_sublimation = 1.989000e+06
NH3.rho_liquid_BoilingPoint = 6.820000e+02
NH3.rho_liquid_TriplePoint = 7.342000e+02
NH3.rho_solid = 8.226000e+02
NH3.cp = 2.060000e+03
NH3.gamma = 1.309000e+00
NH3.MolecularWeight = 1.703100e+01
NH3.name = 'Ammonia' 
NH3.formula = 'NH3' 
NH3.L_vaporization=1.658000e+06
NH3.rho_liquid=7.342000e+02
#------------------------

#------------------------
#Synonyms for volatiles
water   = H2O
methane = CH4
co2     = CO2
co      = CO
n2      = N2
o2      = O2
o3      = O3
h2      = H2
he      = He
nh3     = NH3

#Make a list of all the gases
#
#This clever little fragment uses the fact that
#Python can execute any string as a Python statement,
#in order to find all the gases and build a list of them.
#I don't know if there is a more straightforward way to
#get a list of all the objects of a certain type, but this
#works.  Some of the trickery below is needed because
#the dir() command returns a list of strings, which
#give the names of the objects. It doesn't give the objects
#themselves.
gases = []
for ob in dir():
    exec('isGas=isinstance('+ob+',gas)')
    if isGas:
        exec('gases.append('+ob+')')

#Update all the gases
for gas1 in gases:
    gas1.update()
#
#
#----------------Radiation related functions-------------

#Planck function (of frequency)

def B(nu,T):
    u = min(h*nu/(k*T),500.) #To prevent overflow
    return (2.*h*nu**3/c**2)/(math.exp(u)-1.)
  
#
""" Inefficient, better to integrate than to sum
def B(nu,T):
    Bsum=0.
    for i in range(len(nu)):
        u = min(h*nu[i]/(k*T),500.) #To prevent overflow
        Bi=(2.*h*nu[i]**3/c**2)/(np.exp(u)-1.) # np.exp can take arrays as arguments, math.exp cannot.
        Bsum+=Bi
    return Bsum # Sum over all frequencies in freq(lam), should be close to sigma*T**4.
"""
def dB(nu,T):
    u = min(h*nu/(k*T),500.) #To prevent overflow
    dB=(2*h**2*nu**4/(c**2*k*T**2))*(math.exp(u)/(math.exp(u)-1)**2) # np.exp can take arrays as arguments, math.exp cannot.
    return dB

#----------Saturation Vapor Pressure functions---------------
#

#Saturation vapor pressure over ice (Smithsonian formula)
#    Input: Kelvin. Output: Pascal
def satvpi(T):
   #
   #  Compute es over ice (valid between -153 c and 0 c)
   #  see smithsonian meteorological tables page 350
   #
   #  Original source: GFDL climate model, circa 1995
   esbasi =    6107.1
   tbasi =     273.16
   #
   aa  = -9.09718 *(tbasi/T-1.0)
   b   = -3.56654 *math.log10(tbasi/T)
   c   =  0.876793*(1.0-T/tbasi)
   e   = math.log10(esbasi)
   esice = 10.**(aa+b+c+e)
   return .1*esice  #Convert to Pascals

#Saturation vapor pressure over liquid water (Smithsonian formula)
#    Input: Kelvin. Output: Pascal
def satvpw(T):
   #  compute es over liquid water between -20c and freezing.
   #  see smithsonian meteorological tables page 350.
   #
   #  Original source: GFDL climate model, circa 1995
   esbasw = 1013246.0
   tbasw =     373.16
   #
   aa  = -7.90298*(tbasw/T-1)
   b   =  5.02808*math.log10(tbasw/T)
   c   = -1.3816e-07*(  10.**( ((1-T/tbasw)*11.344)-1 )  )
   d   =  8.1328e-03*(  10.**( ((tbasw/T-1)*(-3.49149))-1)  )
   e   = math.log10(esbasw)
   esh2O  = 10.**(aa+b+c+d+e)
   return .1*esh2O  #Convert to Pascals

# An alternate formula for saturation vapor pressure over liquid water
def satvpw_Heymsfield(T):
  ts=373.16
  sr=3.0057166
# Vapor pressure over water. Heymsfield formula
  ar  = ts/T
  br  = 7.90298*(ar-1.)
  cr  = 5.02808*math.log10(ar);
  dw  = (1.3816E-07)*(10.**(11.344*(1.-1./ar))-1.)
  er  = 8.1328E-03*((10.**(-(3.49149*(ar-1.))) )-1.)
  vp = 10.**(cr-dw+er+sr-br)
  vp=vp*1.0e02
  return(vp)

def satvpg(T):
#This is the saturation vapor pressure computation used in the
#GFDL climate model.  It blends over from water saturation to
#ice saturation as the temperature falls below 0C.
   if ((T-273.16) <  -20.):
      return  satvpi(T)
   if ( ((T-273.16) >= -20.)&((T-273.16)<=0.)):
      return 0.05*(273.16-T)*satvpi(T) + 0.05*(T-253.16)*satvpw(T)
   if ((T-273.16)>0.):
       return satvpw(T)

#Saturation vapor pressure for any substance, computed using
#the simplified form of Clausius-Clapeyron assuming the perfect
#gas law and constant latent heat
def satvps(T,T0,e0,MolecularWeight,LatentHeat):
  Rv=Rstar/MolecularWeight 
  return e0*math.exp(-(LatentHeat/Rv)*(1./T - 1./T0))

#This example shows how to simplify the use of the simplified
#saturation vapor pressure function, by setting up an object
#that stores the thermodynamic data needed, so it doesn't have
#to be re-entered each time.  Because of the __call__ method,
#once the object is created, it can be invoked like a regular
#function.
#
#Usage example:
# To set up a function e(T) that approximates the saturation
# vapor presure for a substance which has a latent heat of
# 2.5e6 J/kg, a molecular weight of 18 and has vapor pressure
# 3589. Pa at a temperature of 300K, create the function using:
#
#         e = satvps_function(300.,3589.,18.,2.5e6)
#
# and afterward you can invoke it simply as e(T), where T
# is whatever temperature you want to evaluate it for.
#
#Alternately, satvps_function can be called with a gas object
#as the first argument, e.g.
#        e = satvps_function(phys.CO2)
#
#If no other arguments are given, the latent heat of sublimation
#will be used when e(T) is called for temperatures below the triple
#point, and the latent heat of vaporization will be used for
#temperatures above the triple point. To allow you to force
#one or the other latent heats to be used, satvps_function takes
#an optional second argument when the first argument is a gas
#object.  Thus,
#       e = satvps_function(phys.CO2,'ice')
#will always use the latent heat of sublimation, regardless of T,
#while  e = satvps_function(phys.CO2,'liquid') will always use
#the latent heat of vaporization.
class satvps_function:
    def __init__(self,Gas_or_T0,e0_or_iceFlag=None,MolecularWeight=None,LatentHeat=None):
        #Check if the first argument is a gas object. If not, assume
        #that the arguments give T0, e0, etc. as numbers
        self.iceFlag = e0_or_iceFlag
        if isinstance(Gas_or_T0,gas):
            self.gas = Gas_or_T0
            self.M = Gas_or_T0.MolecularWeight
            self.T0 = Gas_or_T0.TriplePointT
            self.e0 = Gas_or_T0.TriplePointP
            if self.iceFlag == 'ice':
                self.L = Gas_or_T0.L_sublimation
            elif self.iceFlag == 'liquid':
                self.L = Gas_or_T0.L_vaporization
            else:
                self.iceFlag = 'switch'
            self.M = Gas_or_T0.MolecularWeight
        else:
            self.L = LatentHeat
            self.M = MolecularWeight
            self.T0 = Gas_or_T0
            self.e0 = e0_or_iceFlag

    def __call__(self,T, water_lookup=False):
        #Decide which latent heat to use
        if self.gas == 'H2O' and water_lookup:
            # Water special case -- use IAPWS steam tables in water_tables.py
            if T > wt.T_tp:
                return wt.lookup('psat', T)
            else:
                if self.iceFlag=='switch':
                    L = self.gas.L_sublimation
                else:
                    L = wt.L_vap[0]
                return satvps(T, self.T0, self.e0, self.M, L)
            
        if self.iceFlag == 'switch':
            if T<self.gas.TriplePointT:
                L = self.gas.L_sublimation
            else:
                L = self.gas.L_vaporization
        else:
            L = self.L
        return satvps(T,self.T0,self.e0,self.M,L)



#Class for computing the moist adiabat for a mixture of
#a condensing and noncondensing gas.
#    **ToDo: Add help strings and documentation
#
#    **ToDo: The way the help strings for gas objects are
#            set up makes the argument help box for the
#            creator useless.  Fix this somehow
#
#**ToDo: Add controls on resolution, top of atmosphere, etc.
#Do we want this to return molar or mass concentration?
#Maybe do both, but have result stored as an attribute
#
class MoistAdiabat:
    '''
    MoistAdiabat is a class which creates a callable object
    used to compute the moist adiabat for a mixture consisting
    of a condensible gas and a noncondensing gas.  The gases
    are specified as gas objects. By default, the condensible
    is water vapor and the noncondensible is modern Earth Air,
    if the gases are not specified.

    Usage:
          To create a function m that computes the moist
          adiabat for the gas Condensible mixed with the gas
          Noncondensible, do
                m = phys.MoistAdiabat(Condensible,Noncondensible)
          For example, to do a mixture of condensible CO2 in
          noncondensing N2, do
                m = phys.MoistAdiabat(phys.CO2,phys.N2)
          Once you have created the function, you give it
          the surface partial pressure of the noncondensible
          and the surface temperature when you call it, and it
          returns arrays consisting of pressure, temperature,
          molar concentration of the condensible, and mass
          specific concentration of the condensible. For example:
                p,T,molarCon,massCon = m(1.e5,300.)
          for a surface noncondensible pressure of 1.e5 Pascal and
          surface temperture of 300K.  The values returned
          are arrays. The pressure returned is total pressure at
          each level (condensible plus noncondensible).  By default,
          the compution chooses the pressure values on which to return
          the results.  For some purposes, you might want the results
          specified on a list of pressures of your own choosing.  The
          computation allows for this, by offering an interpolation
          option which returns the result interpolated to a pressure
          grid of your own choice, which is specified as an optional
          third argument to the function. Thus, to get the
          pressure values on a list consisting of [1000.,5000.,10000.] Pa,
          you would do:
                p,T,molarCon,massCon = m(1.e5,300.,[1000.,5000.,10000.])
          The calculation is still done at high resolution to preserve
          accuracy, but the results are afterward intepolated to the grid
          you want using polynomial interpolation. For your convenience,
          the pressure returned on the left hand side is a copy of
          the pressure list you specified as input.
    '''
    def __init__(self,condensible=H2O,noncon = air):
        self.condensible = condensible
        self.noncon = noncon
        #Set up saturation vapor pressure function
        self.satvp = satvps_function(condensible)
        #Set up thermodynamic constants
        self.eps = condensible.MolecularWeight/noncon.MolecularWeight
        self.L = condensible.L_vaporization
        self.Ra = noncon.R
        self.Rc = condensible.R
        self.cpa = noncon.cp
        self.cpc = condensible.cp
        #Set up derivative function for integrator
        def slope(logpa,logT):
            pa = math.exp(logpa)
            T = math.exp(logT)
            qsat = self.eps*(self.satvp(T)/pa)
            num = (1. + (self.L/(self.Ra*T))*qsat)*self.Ra
            den = self.cpa + (self.cpc + (self.L/(self.Rc*T) - 1.)*(self.L/T))*qsat
            return num/den
        self.slope = slope
        # self.ptop = 1000. #Default top of atmosphere
        self.step = -.05 #Default step size for integration
    def __call__(self,ps,Ts,ptop,pgrid=[]):
        #Initial conditions
        step = self.step  #Step size for integration
        # ptop = self.ptop #Where to stop integration
        #
        logpa = math.log(ps)
        logT = math.log(Ts)
        ad = integrator(self.slope,logpa,logT,step)
        #Initialize lists to save results
        pL = [math.exp(logpa) + self.satvp(math.exp(logT))]
        molarConL = [self.satvp(math.exp(logT))/pL[0]]
        TL = [math.exp(logT)]
        #Integration loop
        p = 1.e30 #Dummy initial value, to get started
        while p > ptop:
                ans = ad.next()
                pa = math.exp(ans[0])
                T = math.exp(ans[1])
                p = pa+self.satvp(T)
                pL.append(p)
                molarConL.append(self.satvp(T)/p)
                TL.append(T)
        #Numeric.array turns lists into arrays that one
        #can do arithmetic on.
        pL = numpy.array(pL)
        TL = numpy.array(TL)
        molarConL = numpy.array(molarConL)
        #Now compute mass specific concentration
        Mc = self.condensible.MolecularWeight
        Mnc = self.noncon.MolecularWeight
        Mbar = molarConL*Mc +(1.-molarConL)*Mnc
        qL = (Mc/Mbar)*molarConL
        #
        #The else clause below interpolates to a
        #specified pressure array pgrid, if desired.
        # interp is a class defined in ClimateUtilities
        #which creates a callable object which acts like
        #an interpolation function for the listed data give
        #as arguments.
        if not list(pgrid):
          return pL,TL,molarConL,qL
        else:
          T1 = interp(pL,TL)
          mc1 = interp(pL,molarConL)
          q1 = interp(pL,qL)
          T = numpy.array([T1(pp) for pp in pgrid])
          mc = numpy.array([mc1(pp) for pp in pgrid])
          q = numpy.array([q1(pp) for pp in pgrid])
          return numpy.array(pgrid),T, mc, q







import os
from pathlib import Path
import logging

import platformdirs
from osfclient.api import OSF

log = logging.getLogger("fwl."+__name__)

FWL_DATA_DIR = Path(os.environ.get('FWL_DATA', platformdirs.user_data_dir('fwl_data')))

log.info(f'FWL data location: {FWL_DATA_DIR}')

basic_list = (
        "Dayspring/256",
        "Frostflow/256",
        "Legacy",
        "Mallard",
        "Oak",
        "Reach",
        )

def download_folder(*, storage, folders: list[str], data_dir: Path):
    """
    Download a specific folder in the OSF repository

    Inputs :
        - storage : OSF storage name
        - folders : folder names to download
        - data_dir : local repository where data are saved
    """
    for file in storage.files:
        for folder in folders:
            if not file.path[1:].startswith(folder):
                continue
            parts = file.path.split('/')[1:]
            target = Path(data_dir, *parts)
            target.parent.mkdir(parents=True, exist_ok=True)
            log.info(f'Downloading {file.path}...')
            with open(target, 'wb') as f:
                file.write_to(f)
            break


def GetFWLData() -> Path:
    """
    Get path to FWL data directory on the disk
    """
    return FWL_DATA_DIR.absolute()

def DownloadStellarSpectra():
    """
    Download stellar spectra
    """
    #project ID of the stellar spectra on OSF
    project_id = '8r2sw'
    folder_name = 'Named'

    osf = OSF()
    project = osf.project(project_id)
    storage = project.storage('osfstorage')

    data_dir = GetFWLData() / "stellar_spectra"
    data_dir.mkdir(parents=True, exist_ok=True)

    if not (data_dir / folder_name).exists():
        print(f"Downloading stellar spectra to {data_dir}")
        download_folder(storage=storage, folders=[folder_name], data_dir=data_dir)


def DownloadSpectralFiles(fname: str="",nband: int=256):
    """
    Download spectral files data

    Inputs :
        - fname (optional) :    folder name, i.e. "/Dayspring"
                                if not provided download all the basic list
        - nband (optional) :    number of band = 16, 48, 256, 4096
                                (only relevant for Dayspring, Frostflow and Honeyside)
    """
    #project ID of the spectral files on OSF
    project_id = 'vehxg'

    #Create spectral file data repository if not existing
    data_dir = GetFWLData() / "spectral_files"
    data_dir.mkdir(parents=True, exist_ok=True)

    #Link with OSF project repository
    osf = OSF()
    project = osf.project(project_id)
    storage = project.storage('osfstorage')

    #If no folder specified download all basic list
    if not fname:
        folder_list = basic_list
    elif fname in ("Dayspring", "Frostflow", "Honeyside"):
        folder_list = [fname + "/" + str(nband)]
    elif fname in ("Kynesgrove","Legacy","Mallard","Oak","Reach","stellar_spectra"):
        folder_list = [fname]
    else:
        raise ValueError(f"Unrecognised folder name: {fname}")

    folders = [folder for folder in folder_list if not (data_dir / folder).exists()]

    if folders:
        print(f"Downloading SOCRATES spectral files to {data_dir}")
        download_folder(storage=storage, folders=folders, data_dir=data_dir)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jan 23 11:36:49 2023

@authors:    
Ryan Boukrouche (RB)
"""

def DryAdj(atm):
    """Dry convective adjustment routine.

    Performs a single iteration only. Originally written by RB. 

    Parameters
    ----------
        atm : atmos
            Atmosphere object

    Returns
    ----------
        atm : atmos
            Atmosphere object with adjusted temperature profile

    """

    T   = atm.tmp
    p   = atm.p
    
    # Rcp is global
    # Downward pass
    for i in range(len(T)-1):
        T1,p1 = T[i],p[i]
        T2,p2 = T[i+1],p[i+1]
        
        # Adiabat slope
        pfact = (p1/p2)**atm.Rcp
        
        # If slope is shallower than adiabat (unstable), adjust to adiabat
        if T1 < T2*pfact:
            Tbar = .5*(T1+T2) # Equal layer masses
                              # Not quite compatible with how
                              # heating is computed from flux
            T2 = 2.*Tbar/(1.+pfact)
            T1 = T2*pfact
            atm.tmp[i]   = T1
            atm.tmp[i+1] = T2
    
    # Upward pass
    for i in range(-2,1,-1):

        T1,p1 = T[i],p[i]
        T2,p2 = T[i+1],p[i+1]
        pfact = (p1/p2)**atm.Rcp

        if T1 < T2*pfact:
            Tbar = .5*(T1+T2) # Equal layer masses
                              # Not quite compatible with how
                              # heating is computed from flux
            T2 = 2.*Tbar/(1.+pfact)
            T1 = T2*pfact
            atm.tmp[i]   = T1
            atm.tmp[i+1] = T2 

    return atm      



#Planetary database
#Source for planetary data, and some of the data on
#the moons, is http://nssdc.gsfc.nasa.gov/planetary/factsheet/

class Planet:
    '''
    A Planet object contains basic planetary data.
    If P is a Planet object, the data are:
           P.name = Name of the planet
           P.a = Mean radius of planet (m)
           P.g = Surface gravitational acceleration (m/s**2)
           P.L = Annual mean solar constant (current) (W/m**2)
           P.albedo Bond albedo (fraction)

           P.rsm = Semi-major axis of orbit about Sun (m)
           P.year = Sidereal length of year (s)
           P.eccentricity =  Eccentricity (unitless)
           P.day = Mean tropical length of day (s)
           P.obliquity = Obliquity to orbit (degrees)
           P.Lequinox = Longitude of equinox (degrees)

           P.Tsbar = Mean surface temperature (K)
           P.Tsmax = Maximum surface temperature (K)

    For gas giants, "surface" quantities are given at the 1 bar level
    '''

    #__repr__ object prints out a help string when help is
    #invoked on the planet object or the planet name is typed
    def __repr__(self):
        line1 =\
        'This planet object contains information on %s\n'%self.name
        line2 = 'Type \"help(Planet)\" for more information\n'
        return line1+line2
    def __init__(self):
        self.name = None #Name of the planet
        self.a = None #Mean radius of planet
        self.g = None #Surface gravitational acceleration
        self.L = None #Annual mean solar constant (current)
        self.albedo = None #Bond albedo

        self.rsm = None #Semi-major axis
        self.year = None #Sidereal length of year
        self.eccentricity = None # Eccentricity
        self.day = None #Mean tropical length of day
        self.obliquity = None #Obliquity to orbit
        self.Lequinox = None #Longitude of equinox

        self.Tsbar = None #Mean surface temperature
        self.Tsmax = None #Maximum surface temperature

#----------------------------------------------------
Mercury = Planet()
Mercury.name = 'Mercury' #Name of the planet
Mercury.a = 2.4397e6 #Mean radius of planet
Mercury.g = 3.70 #Surface gravitational acceleration
Mercury.albedo = .119 #Bond albedo
Mercury.L = 9126.6 #Annual mean solar constant (current)
#
Mercury.rsm = 57.91e9 #Semi-major axis
Mercury.year = 87.969*24.*3600. #Sidereal length of year
Mercury.eccentricity = .2056 # Eccentricity
Mercury.day = 4222.6*3600. #Mean tropical length of day
Mercury.obliquity = .01 #Obliquity to orbit (deg)
Mercury.Lequinox = None #Longitude of equinox (deg)
#
Mercury.Tsbar = 440. #Mean surface temperature
Mercury.Tsmax = 725. #Maximum surface temperature

#----------------------------------------------------
Venus = Planet()
Venus.name = 'Venus' #Name of the planet
Venus.a = 6.0518e6 #Mean radius of planet
Venus.g = 8.87 #Surface gravitational acceleration
Venus.albedo = .750 #Bond albedo
Venus.L = 2613.9 #Annual mean solar constant (current)
#
Venus.rsm = 108.21e9 #Semi-major axis
Venus.year = 224.701*24.*3600. #Sidereal length of year
Venus.eccentricity = .0067 # Eccentricity
Venus.day = 2802.*3600. #Mean tropical length of day
Venus.obliquity = 177.36 #Obliquity to orbit (deg)
Venus.Lequinox = None #Longitude of equinox (deg)
#
Venus.Tsbar = 737. #Mean surface temperature
Venus.Tsmax = 737. #Maximum surface temperature

#----------------------------------------------------
Earth = Planet()
Earth.name = 'Earth' #Name of the planet
Earth.a = 6.371e6 #Mean radius of planet
Earth.g = 9.798 #Surface gravitational acceleration
Earth.albedo = .306 #Bond albedo
Earth.L = 1367.6 #Annual mean solar constant (current)
#
Earth.rsm = 149.60e9 #Semi-major axis
Earth.year = 365.256*24.*3600. #Sidereal length of year
Earth.eccentricity = .0167 # Eccentricity
Earth.day = 24.000*3600. #Mean tropical length of day
Earth.obliquity = 23.45 #Obliquity to orbit (deg)
Earth.Lequinox = None #Longitude of equinox (deg)
#
Earth.Tsbar = 288. #Mean surface temperature
Earth.Tsmax = None #Maximum surface temperature

#----------------------------------------------------
Mars = Planet()
Mars.name = 'Mars' #Name of the planet
Mars.a = 3.390e6 #Mean radius of planet
Mars.g = 3.71 #Surface gravitational acceleration
Mars.albedo = .250 #Bond albedo
Mars.L = 589.2 #Annual mean solar constant (current)
#
Mars.rsm = 227.92e9 #Semi-major axis
Mars.year = 686.98*24.*3600. #Sidereal length of year
Mars.eccentricity = .0935 # Eccentricity
Mars.day = 24.6597*3600. #Mean tropical length of day
Mars.obliquity = 25.19 #Obliquity to orbit (deg)
Mars.Lequinox = None #Longitude of equinox (deg)
#
Mars.Tsbar = 210. #Mean surface temperature
Mars.Tsmax = 295. #Maximum surface temperature

#----------------------------------------------------
Jupiter = Planet()
Jupiter.name = 'Jupiter' #Name of the planet
Jupiter.a = 69.911e6 #Mean radius of planet
Jupiter.g = 24.79 #Surface gravitational acceleration
Jupiter.albedo = .343 #Bond albedo
Jupiter.L = 50.5 #Annual mean solar constant (current)
#
Jupiter.rsm = 778.57e9 #Semi-major axis
Jupiter.year = 4332.*24.*3600. #Sidereal length of year
Jupiter.eccentricity = .0489 # Eccentricity
Jupiter.day = 9.9259*3600. #Mean tropical length of day
Jupiter.obliquity = 3.13 #Obliquity to orbit (deg)
Jupiter.Lequinox = None #Longitude of equinox (deg)
#
Jupiter.Tsbar = 165. #Mean surface temperature
Jupiter.Tsmax = None #Maximum surface temperature

#----------------------------------------------------
Saturn = Planet()
Saturn.name = 'Saturn' #Name of the planet
Saturn.a = 58.232e6 #Mean radius of planet
Saturn.g = 10.44 #Surface gravitational acceleration
Saturn.albedo = .342 #Bond albedo
Saturn.L = 14.90 #Annual mean solar constant (current)
#
Saturn.rsm = 1433.e9 #Semi-major axis
Saturn.year = 10759.*24.*3600. #Sidereal length of year
Saturn.eccentricity = .0565 # Eccentricity
Saturn.day = 10.656*3600. #Mean tropical length of day
Saturn.obliquity = 26.73 #Obliquity to orbit (deg)
Saturn.Lequinox = None #Longitude of equinox (deg)
#
Saturn.Tsbar = 134. #Mean surface temperature
Saturn.Tsmax = None #Maximum surface temperature

#----------------------------------------------------
Uranus = Planet()
Uranus.name = 'Uranus' #Name of the planet
Uranus.a = 25.362e6 #Mean radius of planet
Uranus.g = 8.87 #Surface gravitational acceleration
Uranus.albedo = .300 #Bond albedo
Uranus.L = 3.71 #Annual mean solar constant (current)
#
Uranus.rsm = 2872.46e9 #Semi-major axis
Uranus.year = 30685.4*24.*3600. #Sidereal length of year
Uranus.eccentricity = .0457 # Eccentricity
Uranus.day = 17.24*3600. #Mean tropical length of day
Uranus.obliquity = 97.77 #Obliquity to orbit (deg)
Uranus.Lequinox = None #Longitude of equinox (deg)
#
Uranus.Tsbar = 76. #Mean surface temperature
Uranus.Tsmax = None #Maximum surface temperature


#----------------------------------------------------
Neptune = Planet()
Neptune.name = 'Neptune' #Name of the planet
Neptune.a = 26.624e6 #Mean radius of planet
Neptune.g = 11.15 #Surface gravitational acceleration
Neptune.albedo = .290 #Bond albedo
Neptune.L = 1.51 #Annual mean solar constant (current)
#
Neptune.rsm = 4495.06e9 #Semi-major axis
Neptune.year = 60189.0*24.*3600. #Sidereal length of year
Neptune.eccentricity = .0113 # Eccentricity
Neptune.day = 16.11*3600. #Mean tropical length of day
Neptune.obliquity = 28.32 #Obliquity to orbit (deg)
Neptune.Lequinox = None #Longitude of equinox (deg)
#
Neptune.Tsbar = 72. #Mean surface temperature
Neptune.Tsmax = None #Maximum surface temperature

#----------------------------------------------------
Pluto = Planet()
Pluto.name = 'Pluto' #Name of the planet
Pluto.a = 1.195e6 #Mean radius of planet
Pluto.g = .58 #Surface gravitational acceleration
Pluto.albedo = .5 #Bond albedo
Pluto.L = .89 #Annual mean solar constant (current)
#
Pluto.rsm = 5906.e9 #Semi-major axis
Pluto.year = 90465.*24.*3600. #Sidereal length of year
Pluto.eccentricity = .2488 # Eccentricity
Pluto.day = 153.2820*3600. #Mean tropical length of day
Pluto.obliquity = 122.53 #Obliquity to orbit (deg)
Pluto.Lequinox = None #Longitude of equinox (deg)
#
Pluto.Tsbar = 50. #Mean surface temperature
Pluto.Tsmax = None #Maximum surface temperature



#Selected moons

#----------------------------------------------------
Moon = Planet()
Moon.name = 'Moon' #Name of the planet
Moon.a = 1.737e6 #Mean radius of planet
Moon.g = 1.62 #Surface gravitational acceleration
Moon.albedo = .11 #Bond albedo
Moon.L = 1367.6 #Annual mean solar constant (current)
#
Moon.rsm = Earth.rsm #Semi-major axis
Moon.year = Earth.year #Sidereal length of year
Moon.eccentricity = None # Eccentricity
Moon.day = 28.*24.*3600. #Mean tropical length of day (approx)
Moon.obliquity = None #Obliquity to orbit (deg)
Moon.Lequinox = None #Longitude of equinox (deg)
#
Moon.Tsbar = None #Mean surface temperature
Moon.Tsmax = 400. #Maximum surface temperature
Moon.Tsmin = 100. #Minimum surface temperature

Titan = Planet()
Titan.name = 'Titan' #Name of the planet
Titan.a = 2.575e6 #Mean radius of planet
Titan.g = 1.35 #Surface gravitational acceleration
Titan.L = Saturn.L #Annual mean solar constant (current)
Titan.albedo = .21 #Bond albedo (Not yet updated from Cassini)
#
Titan.rsm = None #Semi-major axis
Titan.year = Saturn.year #Sidereal length of year
Titan.eccentricity = Saturn.eccentricity # Eccentricity ABOUT SUN
Titan.day = 15.9452*24.*3600. #Mean tropical length of day
Titan.obliquity = Saturn.obliquity #Obliquity to plane of Ecliptic
                                   #(Titan's rotation axis approx parallel
                                   # to Saturn's
Titan.Lequinox = Saturn.Lequinox #Longitude of equinox
#
Titan.Tsbar = 95. #Mean surface temperature
Titan.Tsmax = None #Maximum surface temperature

Europa = Planet()
Europa.name = 'Europa' #Name of the planet
Europa.a = 1.560e6 #Mean radius of planet
Europa.g = 1.31 #Surface gravitational acceleration
Europa.L = Jupiter.L #Annual mean solar constant (current)
Europa.albedo = .67 #Bond albedo
#
Europa.rsm = Jupiter.rsm #Semi-major axis
Europa.year = Jupiter.year #Sidereal length of year
Europa.eccentricity = Jupiter.eccentricity # Eccentricity
Europa.day = 3.551*24.*3600. #Mean tropical length of day
Europa.obliquity = Jupiter.obliquity #Obliquity to plane of ecliptic
Europa.Lequinox = None #Longitude of equinox
#
Europa.Tsbar = 103. #Mean surface temperature
Europa.Tsmax = 125. #Maximum surface temperature

Triton = Planet()
Triton.name = 'Triton' #Name of the planet
Triton.a = 2.7068e6/2. #Mean radius of planet
Triton.g = .78 #Surface gravitational acceleration
Triton.L = Neptune.L #Annual mean solar constant (current)
Triton.albedo = .76 #Bond albedo
#
Triton.rsm = Neptune.rsm #Semi-major axis
Triton.year = Neptune.year #Sidereal length of year
Triton.eccentricity = Neptune.eccentricity # Eccentricity about Sun
Triton.day = 5.877*24.*3600. #Mean tropical length of day
                             #Triton's rotation is retrograde
Triton.obliquity = 156. #Obliquity to ecliptic **ToDo: Check this.
                        #Note: Seasons are influenced by the inclination
                        #of Triton's orbit? (About 20 degrees to
                        #Neptune's equator
Triton.Lequinox = None #Longitude of equinox
#
Triton.Tsbar = 34.5 #Mean surface temperature
                    #This is probably a computed blackbody
                    #temperature, rather than an observation
Triton.Tsmax = None #Maximum surface temperature



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jan 23 12:06:35 2023

@authors: 
Tim Lichtenberg (TL)    
Ryan Boukrouche (RB)
"""

import numpy as np
import janus.utils.GeneralAdiabat as ga # Moist adiabat with multiple condensibles
    
# Dry adiabat profile
def dry_adiabat_atm(atm):

    R_universal = 8.31446261815324              # Universal gas constant, J.K-1.mol-1

    # Calculate cp from molar concentrations
    cp_mix = 0.
    for vol in atm.vol_list.keys():
        cp_mix += atm.vol_list[vol] * ga.cpv(vol, atm.ts)

    # Calculate dry adiabat slope
    atm.Rcp = R_universal / cp_mix

    # Calculate dry adiabat temperature profile for staggered nodes (from ground up)
    for idx, prsl in enumerate(atm.pl):
        atm.tmpl[idx] = atm.ts * ( prsl / atm.ps ) ** ( atm.Rcp )

    # Interpolate temperature from staggered nodes
    atm.tmp = np.interp(atm.p, atm.pl, atm.tmpl)

    return atm




# The atmosphere class
from .atmosphere_column import *

# Data download
from .data import DownloadSpectralFiles
from .data import DownloadStellarSpectra

# Socrates utility module
from .socrates import CleanOutputDir

# Stellar Spectrum utilities
from .StellarSpectrum import InsertStellarSpectrum, PrepareStellarSpectrum

# Read spectral file
from .ReadSpectralFile import ReadBandEdges

#General adiabat module
from .GeneralAdiabat import plot_adiabats



import numpy as np
import janus.utils.phys as phys

import logging 
log = logging.getLogger("fwl."+__name__)


def gravity( m, r ):
    g = phys.G*m/r**2
    return g

def AtmosphericHeight(atm, m_planet, r_planet):

    z_profile       = np.zeros(len(atm.p))
    grav_s          = gravity( m_planet, r_planet )

    # Reverse arrays to go from high to low pressure
    atm.p   = atm.p[::-1]
    atm.tmp = atm.tmp[::-1]
    for vol in atm.vol_list.keys():
        atm.x_gas[vol] = atm.x_gas[vol][::-1]

    atm.height_error = False 
    for n in range(0, len(z_profile)-1):

        # Gravity with height
        grav_z = grav_s * ((r_planet)**2) / ((r_planet + z_profile[n])**2)

        # Mean molar mass depending on mixing ratio
        mean_molar_mass = 0
        for vol in atm.vol_list.keys():
            mean_molar_mass += phys.molar_mass[vol]*atm.x_gas[vol][n]

        # Use hydrostatic equation to get height difference
        dz = phys.R_gas * atm.tmp[n] / (mean_molar_mass * grav_z * atm.p[n]) * (atm.p[n] - atm.p[n+1]) 
        
        # Next height
        z_profile[n+1] = z_profile[n] + dz

        # Check if heights are very large.
        # This implies that the hydrostatic/gravity integration failed.
        if z_profile[n+1] > 1.0e8:
            atm.height_error = True 
            log.warning("Hydrostatic integration blew up. Setting dummy values for height")
            break

    # Set dummy values 
    if atm.height_error:
        z_profile = np.linspace(0.0, 1000.0, len(atm.p))
        
    # Reverse arrays again back to normal
    atm.p   = atm.p[::-1]
    atm.tmp = atm.tmp[::-1]
    for vol in atm.vol_list.keys():
        atm.x_gas[vol] = atm.x_gas[vol][::-1]
    z_profile = z_profile[::-1]

    return z_profile



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec 6 11:22:05 2023

@authors:    
Ryan Boukrouche (RB)
"""

import numpy as np
import janus.utils.GeneralAdiabat as ga # Moist adiabat with multiple condensibles

def simple_cloud(atm):
    """Simple water cloud routine.

    Define the three SOCRATES cloud input arrays with user-defined values where cloud condensation occurs. These values could be made to vary with height. 

    Parameters
    ----------
        atm : atmos
            Atmosphere object

    """

    nlev = len(atm.p)
    Tmid = atm.tmp.copy()

    if (np.sum(atm.x_cond['H2O']) < 1.0e-30): # If x_cond = 0, no condensation scheme to provide lwm

        for i in range(nlev-1): 
            pp_h2o = atm.p_vol["H2O"][i]
            if (pp_h2o < 1e-10):
                continue
            
            if (Tmid[i] <= ga.Tdew('H2O',pp_h2o)+1.0e-30):
                atm.re[i]   = atm.effective_radius
                atm.lwm[i]  = atm.liquid_water_fraction
                atm.clfr[i] = atm.cloud_fraction
            else:
                atm.re[i]   = 0.0
                atm.lwm[i]  = 0.0
                atm.clfr[i] = 0.0

    else: # a condensation scheme is providing x_cond

        for i in range(nlev-1): 
            pp_h2o = atm.p_vol["H2O"][i]
            if (pp_h2o < 1e-10):
                continue

            if (atm.x_cond['H2O'][i] > 0.0):
                atm.re[i]   = atm.effective_radius
                atm.lwm[i]  = atm.x_cond['H2O'][i]
                atm.clfr[i] = atm.cloud_fraction
            else:
                atm.re[i]   = 0.0
                atm.lwm[i]  = 0.0
                atm.clfr[i] = 0.0

    return atm



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jan 31 10:03:32 2023

@authors: 
Mark Hammond (MH)  
Ryan Boukrouche (RB)
"""

def simple_boundary_tend(npz,pt,ts,t_dt,ts_dt,mix_coeff_atmos=1e6,mix_coeff_surf=1e6):

    t_dt[npz] += (ts + (ts - pt[npz]))/mix_coeff_atmos
    ts_dt -= (ts - pt[npz])/mix_coeff_surf
    
    return t_dt, ts_dt


#===================================================================
#Computes fluxes and heating rates for the grey gas model.
#The fluxes are computed as a function of p/ps, given net optical
#thickness of the atmosphere tauinf .
#
#Modified to return dimensional heat and flux for use with
#the time-stepping radiative-convective model
#===================================================================

#Data on section of text which this script is associated with
#MODIFIED
Chapter = '4.**'
Figure = 'fig:AllTropNetIRFluxGrey'
#
#This is also the solution script for
Problem = '{Workbook:RadBalance2:PressBroadenedHeating}'
#This script can also be modified to use for the problem
# '{Workbook:RadBalance2:StratTropOLRGrey}'

import janus.utils.phys as phys
import math
# from ClimateUtilities import *


#Grey gas transmission function.
#tauinf is a global
def Trans(tau1,tau2):
    return math.exp(-abs(tau1-tau2))

#Integrand for upward or downward flux. Note that
#the Schwartzschild integral is written here as an integral
#over p/ps, and correspondingly the gradient of T is written as
#dT/d(p/ps). The solution is written in the form of
#Eq. (4.13) (in First Edition).
#

def integrand(ppsp,params):
    #Without pressure broadening
    if PressureBroadening:
        tau1 = tauInf*(1.-ppsp**2)
        tau2 = tauInf*(1.-params.pps**2)
    else:
        tau1 = tauInf*(1.-ppsp)
        tau2 = tauInf*(1. - params.pps)
    Tfun = params.Tfun
    dTdp = (Tfun(ppsp+.01)-Tfun(ppsp-.01))/.02
    return Trans(tau1,tau2)*4.*phys.sigma*Tfun(ppsp)**3*dTdp

def Iplus(pps,Tfun,Tg):
    params = Dummy()
    params.pps = pps
    params.Tfun = Tfun
    Ts = Tfun(1.)
    limit = min(1.,pps+10./tauInf)
    quad = romberg(integrand,10)
    if PressureBroadening:
        tau = tauInf*(1.-pps**2)
    else:
        tau = tauInf*(1.-pps)
    BddTerm = (phys.sigma*Tg**4 - phys.sigma*Ts**4)*Trans(0.,tau)
    return quad([pps,limit],params,.1)+ phys.sigma*Tfun(pps)**4 +BddTerm

def Iminus(pps,Tfun,Tg):
    params = Dummy()
    params.pps = pps
    params.Tfun = Tfun
    limit = max(0.,pps-10./tauInf)
    quad = romberg(integrand,10)
    Tstrat = Tfun(0.)
    if PressureBroadening:
        tau = tauInf*(1.-pps**2)
    else:
        tau = tauInf*(1.-pps)
    return quad([pps,0.],params,.1)+ phys.sigma*Tfun(pps)**4 - phys.sigma*Tstrat**4*Trans(tau,tauInf)

#Return dimensional flux and heating
#pList is dimensional pressure.
#Moisture q not used for gray gas
#

#Pressure in mb, for consistency with ccmrad
def radcomp(pList,TList,Tg,q):
    ppsL = pList/pList[-1] #p/psurf
    Tfun = interp(ppsL,TList)
    Ip = numpy.array([Iplus(pps,Tfun,Tg) for pps in ppsL])
    Im = numpy.array([Iminus(pps,Tfun,Tg) for pps in ppsL])
    flux = Ip-Im
    heat = -2.*phys.sigma*TList**4 + (Ip+Im)
    #
    #Re-dimensionalize heating to K/day
    if PressureBroadening :
        kappa = 2.*tauInf*g/(100.*pList[-1])
    else:
        kappa = tauInf*g/(100.*pList[-1])
    heat = heat*(kappa/cp)*24.*3600. #K/day
    return flux,heat

#These are all globals
tauInf = 10.
PressureBroadening = True

#Gravity and cp (for diminensionalization)
g = 10.
cp = 1000.



import click


@click.group()
def cli():
    pass


@click.group()
def download():
    """Download data and dependencies"""
    pass


@click.command()
@click.option('-n', '--name', 'fname', type=str, help='Name of the spectra')
@click.option('-b', '--band', 'nband', type=int, help='Number of the band', default=256)
def spectral(**kwargs):
    """Download spectral files

    By default, download all files.
    """
    from .utils.data import DownloadSpectralFiles

    DownloadSpectralFiles(**kwargs)


@click.command()
def stellar():
    """Download stellar spectra"""
    from .utils.data import DownloadStellarSpectra

    DownloadStellarSpectra()


@click.command()
@click.option('-r', '--ref', type=str, default='main', help='Git reference (hash) to download')
def socrates(**kwargs):
    """Download SOCRATES code"""
    from .socrates import download_socrates

    download_socrates(**kwargs)


@click.command()
def env():
    """Show environment variables and locations"""
    from janus.socrates import SOCRATES_DIR
    from janus.utils.data import FWL_DATA_DIR

    click.echo(f'RAD_DIR location: {SOCRATES_DIR}')
    click.echo(f'FWL_DATA location: {FWL_DATA_DIR}')


cli.add_command(download)
download.add_command(spectral)
download.add_command(stellar)
download.add_command(socrates)
cli.add_command(env)

if __name__ == '__main__':
    cli()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jan 23 11:59:40 2023

@authors: 
Tim Lichtenberg (TL)    
Ryan Boukrouche (RB)
Harrison Nicholls (HN)
"""

import numpy as np

import logging 
log = logging.getLogger("fwl."+__name__)


def find_tropopause(atm_moist, dynamic: bool, verbose=True):
    """Computes tropopause location via two methods: dynamically based on heating rates, or by a set temperature value.

    Parameters
    ----------
        atm_moist : atmos
            Atmosphere (moist) object from atmosphere_column.py
        dynamic : bool
            Calculate dynamically with heating rates? If not, use fixed value of trppT.
            
    """

    # Heating criterion
    if dynamic:

        log.debug("TROPOPAUSE SET BY HEATING")

        # Find tropopause index
        trpp_idx   = 0 
        signchange = ((np.roll(np.sign(atm_moist.net_heating), 1) - np.sign(atm_moist.net_heating)) != 0).astype(int)[1:]
        signchange_indices = np.nonzero(signchange)[0]

        # Criteria for "significant " heating
        DeltaT_max_sign  = 50.
        DeltaT_at_trpp   = 30.
        DeltaT_mean_sign = 10.
        dZ_strato        = round(len(atm_moist.net_heating)*0.02)
        
        # If heating sign change below TOA -> tropopause
        if np.size(signchange_indices) > 0 and np.max(atm_moist.net_heating) > DeltaT_max_sign:

            # First guess: maximum heating index
            max_heat_idx = np.argmax(atm_moist.net_heating)

            # Lower height while heating still significant
            while atm_moist.net_heating[max_heat_idx] > DeltaT_at_trpp and max_heat_idx < len(atm_moist.p)-1:
                max_heat_idx += 1

            trpp_idx     = max_heat_idx


            # # First guess: uppermost sign change (below TOA)
            # if atm_moist.net_heating[signchange_indices[0]-1] > 0 and np.mean(atm_moist.net_heating[:signchange_indices[0]-1]) > DeltaT_mean_sign:
            #     trpp_idx = signchange_indices[0]

            # # Decrease trpp height (== increase idx) while heating in trpp layer is significant
            # for idx, sgn_idx_top in enumerate(signchange_indices):

            #     if idx < np.size(signchange_indices)-1:
            #         sgn_idx_down = signchange_indices[idx+1]
                
            #         # Check mean and max heating and extent of layer above trpp idx
            #         if np.mean(atm_moist.net_heating[sgn_idx_top:sgn_idx_down]) > DeltaT_mean_sign and np.max(atm_moist.net_heating[sgn_idx_top:sgn_idx_down]) > DeltaT_max_sign and abs(sgn_idx_down-sgn_idx_top) > dZ_strato:
            #             trpp_idx = sgn_idx_down

            # Only consider tropopause if deeper than X% below TOA
            if trpp_idx < dZ_strato: 
                trpp_idx = 0

            # Only consider if mean heating above tropopause significant
            if np.mean(atm_moist.net_heating[0:trpp_idx]) < DeltaT_mean_sign:
                trpp_idx = 0


        # # If heating everywhere (close to star) & heating is significant
        # if np.size(signchange_indices) <= 1 and np.mean(atm_moist.net_heating) > DeltaT_mean_sign:
        #     trpp_idx = np.size(atm_moist.tmp)-1

        # If significant tropopause found or isothermal atmosphere from stellar heating
        if trpp_idx != 0:

            atm_moist.trppidx   = trpp_idx                  # index
            atm_moist.trppP     = atm_moist.pl[trpp_idx]    # pressure 
            atm_moist.trppT     = atm_moist.tmpl[trpp_idx]  # temperature

    # Temperature criterion
    else:

        log.debug("TROPOPAUSE SET BY CONTANT VALUE OF %.2f K"%atm_moist.trppT)

        # Tropopause is triggered
        if np.any(atm_moist.tmpl <= atm_moist.trppT) or np.any(atm_moist.tmp <= atm_moist.trppT):
            # Loop from bottom upwards to find first level where temperature drops below required value
            for i in range(len(atm_moist.tmpl)-1,-1,-1):
                if atm_moist.tmpl[i] < atm_moist.trppT:
                    # Found tropopause, so break loop and set result
                    atm_moist.trppidx = i
                    atm_moist.trppP   = atm_moist.pl[i] 
                    break

            # Make-safe the edge case where trppT > T_surf
            atm_moist.trppidx = int(min(atm_moist.trppidx, len(atm_moist.tmpl)-1))
            atm_moist.trppidx = int(min(atm_moist.trppidx, len(atm_moist.tmp)-1))
            atm_moist.trppP   = atm_moist.pl[atm_moist.trppidx] 

        # Tropopause isn't triggered
        else:
            atm_moist.trppidx   = -1
            atm_moist.trppP     = 1.0e-10

    return atm_moist




'''
Created 01/12/19

@authors: 
Ryan Boukrouche (RB)
Tim Lichtenberg (TL)
RJ Graham (RJ)
This file builds a self-consistent temperature profile from either Li, Ingersoll & Oyafuso 2018 or 
Graham, Boukrouche, Lichtenberg, & Pierrehumbert 2020, from the ground up using the Runge-Kutta 4 scheme 
'''

import time
import numpy as np
import scipy.interpolate as spint
import math
import copy
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import numpy as np
import janus.utils.water_tables as wt

import logging
log = logging.getLogger("fwl."+__name__)

from janus.utils.cp_funcs import *
from janus.utils.ClimateUtilities import *
from janus.utils.atmosphere_column import atmos
import janus.utils.phys as phys

# Color definitions: 
# https://www.codecademy.com/articles/seaborn-design-ii
# https://python-graph-gallery.com/python-colors/
# https://matplotlib.org/tutorials/colors/colormaps.html
# https://chrisalbon.com/python/data_visualization/seaborn_color_palettes/
no_colors   = 7
vol_colors = {
    "O2"             : [mpl.colormaps["Wistia"](i) for i in np.linspace(0,1.0,no_colors+2)],
    "O3"             : [mpl.colormaps["spring"](i) for i in np.linspace(0,1.0,no_colors+2)],
    "S"              : [mpl.colormaps["YlOrBr"](i) for i in np.linspace(0,1.0,no_colors)],
    "H2O": "#027FB1",
    "CO2": "#D24901",
    "H2" : "#008C01",
    "CH4": "#C720DD",
    "CO" : "#D1AC02",
    "N2" : "#870036",
    "S2" : "#FF8FA1",
    "SO2": "#00008B",
    "He" : "#30FF71",
    "NH3": "#675200",
    "mixtures"       : [mpl.colormaps["Set3"](i) for i in np.linspace(0,1.0,no_colors)],
    "H2O-CO2"        : mpl.colormaps["Set3"](1.0/no_colors),
    "CO2-H2O"        : mpl.colormaps["Set3"](1.0/no_colors),
    "H2O-H2"         : mpl.colormaps["Set3"](2.0/no_colors),
    "H2-H2O"         : mpl.colormaps["Set3"](2.0/no_colors),
    "CO-H2"          : mpl.colormaps["Set3"](3.0/no_colors),
    "H2-CO"          : mpl.colormaps["Set3"](3.0/no_colors),
    "H2-CO2"         : mpl.colormaps["Set3"](4.0/no_colors),
    "CO2-H2"         : mpl.colormaps["Set3"](4.0/no_colors),
    "CH4-H2"         : mpl.colormaps["Set3"](5.0/no_colors),
    "H2-CH4"         : mpl.colormaps["Set3"](5.0/no_colors),
    "H2-CH4"         : mpl.colormaps["Set3"](5.0/no_colors),
    "H2-N2"          : mpl.colormaps["Set2"](0.0/no_colors),
    "N2-H2"          : mpl.colormaps["Set2"](0.0/no_colors),
    "CO2-N2"         : mpl.colormaps["Set2"](1.0/no_colors),
    "N2-CO2"         : mpl.colormaps["Set2"](1.0/no_colors),
    "black_1"        : "#000000",
    "black_2"        : "#323232",
    "black_3"        : "#7f7f7f",
    "qgray"          : "#768E95",
    "qgray2"         : "#888888",
    "qblue"          : "#4283A9", # http://www.color-hex.com/color/4283a9
    "qgreen"         : "#62B4A9", # http://www.color-hex.com/color/62b4a9
    "qred"           : "#E6767A",
    "qturq"          : "#2EC0D1",
    "qorange"        : "#ff7f0e",
    "qmagenta"       : "#9A607F",
    "qyellow"        : "#EBB434",
    "qgray_dark"     : "#465559",
    "qblue_dark"     : "#274e65",
    "qgreen_dark"    : "#3a6c65",
    "qred_dark"      : "#b85e61",
    "qturq_dark"     : "#2499a7",
    "qmagenta_dark"  : "#4d303f",
    "qyellow_dark"   : "#a47d24",
    "qgray_light"    : "#acbbbf",
    "qblue_light"    : "#8db4cb",
    "qgreen_light"   : "#a0d2cb",
    "qred_light"     : "#eb9194",
    "qturq_light"    : "#57ccda",
    "qmagenta_light" : "#c29fb2",
    "qyellow_light"  : "#f1ca70",
}

# Volatile Latex names
vol_latex = {
    "H2O"     : r"H$_2$O",
    "CO2"     : r"CO$_2$",
    "H2"      : r"H$_2$" ,
    "CH4"     : r"CH$_4$",
    "CO"      : r"CO",
    "N2"      : r"N$_2$",
    "S"       : r"S",
    "O2"      : r"O$_2$",
    "O3"      : r"O$_3$",
    "He"      : r"He",
    "NH3"     : r"NH$_3$",
    "H2O-CO2" : r"H$_2$O–CO$_2$",
    "H2O-H2"  : r"H$_2$O–H$_2$",
    "H2O-CO"  : r"H$_2$O–CO",
    "H2O-CH4" : r"H$_2$O–CH$_4$",
    "H2O-N2"  : r"H$_2$O–N$_2$",
    "H2O-O2"  : r"H$_2$O–O$_2$",
    "H2-H2O"  : r"H$_2$–H$_2$O",
    "H2-CO"   : r"H$_2$–CO",
    "H2-CH4"  : r"H$_2$–CH$_4$",
    "H2-CO2"  : r"H$_2$–CO$_2$",
    "H2-N2"   : r"H$_2$–N$_2$",
    "H2-O2"   : r"H$_2$-O$_2$",
    "CO2-N2"  : r"CO$_2$–N$_2$",
    "CO2-H2O" : r"CO$_2$–H$_2$O",
    "CO2-CO"  : r"CO$_2$–CO",
    "CO2-CH4"  : r"CO$_2$–CH$_4$",
    "CO2-O2"  : r"CO$_2$–O$_2$",
    "CO2-H2"  : r"CO$_2$–H$_2$",
    "CO-H2O" : r"CO–H$_2$O",
    "CO-CO2" : r"CO–CO$_2$",
    "CO-H2"  : r"CO–H$_2$",
    "CO-CH4" : r"CO–CH$_4$",
    "CO-N2"  : r"CO–N$_2$",
    "CO-O2"  : r"CO–O$_2$",
    "CH4-H2O" : r"CH$_4$–H$_2$O",
    "CH4-CO2" : r"CH$_4$–CO$_2$",
    "CH4-H2"  : r"CH$_4$–H$_2$",
    "CH4-CO"  : r"CH$_4$–CO",
    "CH4-CH4" : r"CH$_4$–CH$_4$",
    "CH4-N2"  : r"CH$_4$–N$_2$",
    "CH4-O2"  : r"CH$_4$–O$_2$",
    "N2-H2O" : r"N$_2$–H$_2$O",
    "N2-CO2" : r"N$_2$–CO$_2$",
    "N2-H2"  : r"N$_2$–H$_2$",
    "N2-CO"  : r"N$_2$–CO",
    "N2-CH4" : r"N$_2$–CH$_4$",
    "N2-N2"  : r"N$_2$–N$_2$",
    "N2-O2"  : r"N$_2$–O$_2$",
    "O2-H2O" : r"O$_2$–H$_2$O",
    "O2-CO2" : r"O$_2$–CO$_2$",
    "O2-H2"  : r"O$_2$–H$_2$",
    "O2-CO"  : r"O$_2$–CO",
    "O2-CH4" : r"O$_2$–CH$_4$",
    "O2-N2"  : r"O$_2$–N$_2$",
    "O2-O2"  : r"O$_2$–O$_2$",
}

def atm_z(atm, idx):

    # Mean temperature below
    #T_mean_down = np.mean( atm.tmp[:idx+1] )

    # # Average temperature weighted by amount of substance?
    # T_mean_down = np.average( atm.tmp[:idx+1], weights=)

    # # Use gas phase mean molar mass
    # atm.mu[idx] = atm.mu[idx]


    mu_c = 0.
    if atm.xc[idx] > 0:
        for vol in atm.vol_list.keys():
            mu_c += atm.x_cond[vol][idx]*phys.molar_mass[vol] / atm.xc[idx]
    
    # Integration
    atm.rho[idx] =  atm.p[idx]/phys.R_gas/atm.tmp[idx] * ( atm.mu[idx] + mu_c*atm.alpha_cloud*atm.xc[idx]/(atm.xv[idx]+atm.xd[idx]) )
    #     cell centres
    dp = atm.p[idx+1]-atm.p[idx]
    dz =  -dp / atm.rho[idx]
    atm.z[idx+1] = atm.z[idx] + dz
    #     cell edges 
    dp = atm.pl[idx+1]-atm.pl[idx]
    dz =  -dp / atm.rho[idx]
    atm.zl[idx+1] = atm.zl[idx] + dz
    if dz<0:
        log.warning("dz  = %g < 0 " % dz)
        log.warning("(dp  = %g)" % dp)
        log.warning("(rho = %g)" % atm.rho[idx])
    
    # Next gravity
    atm.grav_z[idx+1] = atm.grav_s * ((atm.planet_radius)**2) / ((atm.planet_radius+atm.z[idx+1])**2)

    # Calculate scale height in km
    # H = 1e-3 * phys.R_gas * atm.tmp[idx] / (atm.mu[idx] * atm.grav_z[idx])

    return atm

## Saturation vapor pressure [Pa] for given temperature T [K]. 
## Assuming the ideal gas law and a constant latent heat of vaporization. 
## Select the molecule of interest with the switch argument (a string).
def p_sat(switch,T, water_lookup=False): 

    # Force these volatiles to be dry by setting their saturation vapour pressure to be very large
    force_dry = ["H2"]
    if switch in force_dry:
        return 1.0e30

    # Define volatile
    match switch:
        case 'H2O':
            e = phys.satvps_function(phys.water)
        case 'CH4':
            e = phys.satvps_function(phys.methane)
        case 'CO2':
            e = phys.satvps_function(phys.co2)
        case 'CO':
            e = phys.satvps_function(phys.co)
        case 'N2':
            e = phys.satvps_function(phys.n2)
        case 'O2':
            e = phys.satvps_function(phys.o2)
        case 'O3':
            e = phys.satvps_function(phys.o3)
        case 'H2':
            e = phys.satvps_function(phys.h2)
        case 'He':
            e = phys.satvps_function(phys.he)
        case 'NH3':
            e = phys.satvps_function(phys.nh3)   
        case _:
            return 1.0e30
        
    # Return saturation vapor pressure
    svp = e(T,water_lookup=water_lookup)
    return float(svp)


## Dew point temperature [K] given a pressure p [Pa]. Select the molecule of interest with the switch argument (a string).
def Tdew(switch, p): 
    
    # Avoid math error for p = 0
    p = np.max([p, 1e-100])

    if switch == 'H2O':
        p_triple = 611.657 # Pa
        if p > p_triple:
            Tref = 373.15 # K, boiling point of H2O at 1 atm 
            pref = 1e5 # esat('H2O',Tref) returns 121806.3 Pa, should return 1e5    
            L_H2O=phys.water.L_vaporization*phys.water.MolecularWeight*1e-3
        else:
            Tref = 273.15 #K triple point temperature of H2O
            pref = p_triple #triple point pressure of H2O
            L_H2O=phys.water.L_sublimation*phys.water.MolecularWeight*1e-3
        return Tref/(1.-(Tref*phys.R_gas/L_H2O)*math.log(p/pref))
        #return (B(p)/(A(p)-numpy.log10(p/pref)))-C(p)
    if switch == 'CH4':
        Tref = 148.15 # K, arbitrary point (148.15K,esat(148.15K)=9.66bar) on the L/G coexistence curve of methane 
        pref = p_sat('CH4',Tref)
        L_CH4=phys.CH4.L_vaporization*phys.methane.MolecularWeight*1e-3
        return Tref/(1.-(Tref*phys.R_gas/L_CH4)*math.log(p/pref))
    if switch == 'CO2':
        p_triple = 5.11e5
        if p > p_triple:
            Tref = 253. # K, arbitrary point (253K,esat(253K)=20.9bar) on the coexistence curve of CO2 
            pref = p_sat('CO2',Tref)
            L_CO2=phys.CO2.L_vaporization*phys.co2.MolecularWeight*1e-3
        else:
            Tref = 216.58 # K; triple point
            pref = p_triple
            L_CO2 = phys.CO2.L_sublimation*phys.co2.MolecularWeight*1e-3
        return Tref/(1.-(Tref*phys.R_gas/L_CO2)*math.log(p/pref))
    if switch == 'CO':
        Tref = 100. # K, arbitrary point (100K,esat(100K)=4.6bar) on the coexistence curve of CO 
        pref = p_sat('CO',Tref)
        L_CO=phys.CO.L_vaporization*phys.co.MolecularWeight*1e-3
        return Tref/(1.-(Tref*phys.R_gas/L_CO)*math.log(p/pref))
    if switch == 'N2':
        Tref = 98.15 # K, arbitrary point (98.15K,esat(98.15K)=7.9bar) on the coexistence curve of N2 
        pref = p_sat('N2',Tref)
        L_N2=phys.N2.L_vaporization*phys.n2.MolecularWeight*1e-3
        return Tref/(1.-(Tref*phys.R_gas/L_N2)*math.log(p/pref))
    if switch == 'O2':
        Tref = 123.15 # K, arbitrary point (123.15K,esat(123.15K)=21.9bar) on the coexistence curve of O2 
        pref = p_sat('O2',Tref)
        L_O2=phys.O2.L_vaporization*phys.o2.MolecularWeight*1e-3
        return Tref/(1.-(Tref*phys.R_gas/L_O2)*math.log(p/pref))
    if switch == 'O3':
        Tref = 161.85 # K, arbitrary point (161.85K,esat(161.85K)=1.013bar) on the coexistence curve of O3 
        pref = p_sat('O3',Tref)
        L_O3=phys.O3.L_vaporization*phys.o3.MolecularWeight*1e-3
        return Tref/(1.-(Tref*phys.R_gas/L_O3)*math.log(p/pref))
    if switch == 'H2':
        Tref = 23.15 # K, arbitrary point (23.15K,esat(23.15K)=1.7bar) on the coexistence curve of H2 
        pref = p_sat('H2',Tref)
        L_H2=phys.H2.L_vaporization*phys.h2.MolecularWeight*1e-3
        return Tref/(1.-(Tref*phys.R_gas/L_H2)*math.log(p/pref))
    if switch == 'He':
        Tref = 4.22 # K, boiling point of He at 1 atm 
        pref = 1e5 # esat('He',Tref) returns 45196 Pa, should return 1e5
        L_He=phys.He.L_vaporization*phys.he.MolecularWeight*1e-3
        return Tref/(1.-(Tref*phys.R_gas/L_He)*math.log(p/pref))
    if switch == 'NH3':
        Tref = 273.15 # K, arbitrary point (273.15K,esat(273.15K)=8.6bar) on the coexistence curve of NH3 
        pref = p_sat('NH3',Tref)
        L_NH3=phys.NH3.L_vaporization*phys.nh3.MolecularWeight*1e-3
        return Tref/(1.-(Tref*phys.R_gas/L_NH3)*math.log(p/pref))
    
    return 0.0


def get_beta(switch, T, water_lookup=False):
    if (switch == 'H2O') and water_lookup \
        and (T > phys.H2O.TriplePointT) \
        and (T< phys.H2O.CriticalPointT):
        beta = wt.lookup('phase_grad', T)
    else:
        L = L_heat(switch, T)
        beta = L/phys.R_gas/T

    return beta
## Molar latent heat [J mol-1] for gas phase considered given a temperature T [K]. 
## Select the molecule of interest with the switch argument (a string).

def get_T_crit(switch):
    match switch:
        case 'H2O':
            return phys.H2O.CriticalPointT
        case 'CH4':
            return phys.CH4.CriticalPointT
        case 'CO2':
            return phys.CO2.CriticalPointT
        case 'CO':
            return phys.CO.CriticalPointT
        case 'N2':
            return phys.N2.CriticalPointT
        case 'O2':
            return phys.O2.CriticalPointT
        case 'O3':
            return phys.O3.CriticalPointT
        case 'H2':
            return phys.H2.CriticalPointT
        case 'He':
            return phys.He.CriticalPointT
        case 'NH3':
            return phys.NH3.CriticalPointT
        case _:
            return 0.0
        
def L_heat(switch, T, water_lookup=False):

    match switch:
        case 'H2O':
            L_sublimation   = phys.H2O.L_sublimation
            
            if water_lookup:
                L_vaporization  = wt.lookup('L_vap', T)
            else:
                L_vaporization  = phys.H2O.L_vaporization

            MolecularWeight = phys.H2O.MolecularWeight
            T_triple        = phys.H2O.TriplePointT
            T_crit          = phys.H2O.CriticalPointT
        
        case 'CH4':
            L_sublimation   = phys.CH4.L_sublimation
            L_vaporization  = phys.CH4.L_vaporization
            MolecularWeight = phys.CH4.MolecularWeight
            T_triple        = phys.CH4.TriplePointT
            T_crit          = phys.CH4.CriticalPointT
            
        case 'CO2':
            L_sublimation   = phys.CO2.L_sublimation
            L_vaporization  = phys.CO2.L_vaporization
            MolecularWeight = phys.CO2.MolecularWeight
            T_triple        = phys.CO2.TriplePointT
            T_crit          = phys.CO2.CriticalPointT
            
        case 'CO':
            L_sublimation   = phys.CO.L_sublimation
            L_vaporization  = phys.CO.L_vaporization
            MolecularWeight = phys.CO.MolecularWeight
            T_triple        = phys.CO.TriplePointT
            T_crit          = phys.CO.CriticalPointT
                
        case 'N2':
            L_sublimation   = phys.N2.L_sublimation
            L_vaporization  = phys.N2.L_vaporization
            MolecularWeight = phys.N2.MolecularWeight
            T_triple        = phys.N2.TriplePointT
            T_crit          = phys.N2.CriticalPointT
            
        case 'O2':
            L_sublimation   = phys.O2.L_sublimation
            L_vaporization  = phys.O2.L_vaporization
            MolecularWeight = phys.O2.MolecularWeight
            T_triple        = phys.O2.TriplePointT
            T_crit          = phys.O2.CriticalPointT
        
        case 'O3':
            L_sublimation   = phys.O3.L_sublimation # No O3 sublimation
            L_vaporization  = phys.O3.L_vaporization # No triple point L_vap
            MolecularWeight = phys.O3.MolecularWeight
            T_triple        = phys.O3.TriplePointT
            T_crit          = phys.O3.CriticalPointT

        case 'H2':
            L_sublimation   = phys.H2.L_vaporization # No H2 sublimation
            L_vaporization  = phys.H2.L_vaporization
            MolecularWeight = phys.H2.MolecularWeight
            T_triple        = phys.H2.TriplePointT
            T_crit          = phys.H2.CriticalPointT
            
        case 'He':
            L_sublimation   = phys.He.L_vaporization  # No He sublimation
            L_vaporization  = phys.He.L_vaporization
            MolecularWeight = phys.He.MolecularWeight
            T_triple        = phys.He.TriplePointT
            T_crit          = phys.He.CriticalPointT
            
        case 'NH3':
            L_sublimation   = phys.NH3.L_sublimation
            L_vaporization  = phys.NH3.L_vaporization
            MolecularWeight = phys.NH3.MolecularWeight
            T_triple        = phys.NH3.TriplePointT
            T_crit          = phys.NH3.CriticalPointT
        case _:
            L_sublimation   = 0.0
            L_vaporization  = 0.0
            MolecularWeight = 0.0
            T_triple        = 0.0
            T_crit          = 0.0

    # Gas-solid transition
    if T <= T_triple:
        # Conversion from J.kg-1 to J.mol-1 (molecular weight is in g/mol in phys.f90)
        L_heat = L_sublimation*MolecularWeight*1e-3 
    # Gas-liquid transition
    elif T <= T_crit:
        L_heat = L_vaporization*MolecularWeight*1e-3 
    # Super-critical state
    else:
        # Numerical issus with 0.?
        L_heat = 0.         #1e-50
        # L_heat = L_vaporization*MolecularWeight*1e-3 

    # # No latent heat contribution in moist adiabat if below p_sat
    # if P < p_sat(switch, T):
    # # if T > Tdew(switch, psat):
    #     L_heat = 0.

    return L_heat  


satvph2o = phys.satvps_function(phys.H2O)
def slopeRay( logpa, logT ):
    eps     = phys.H2O.MolecularWeight/phys.air.MolecularWeight
    L       = phys.H2O.L_vaporization
    Ra      = phys.air.R
    Rc      = phys.H2O.R
    cpa     = phys.air.cp
    cpc     = phys.H2O.cp
    pa      = math.exp(logpa)
    T       = math.exp(logT)
    qsat    = eps*(satvph2o(T)/pa)
    log.debug('qsat=%.3f'%qsat)
    num     = (1. + (L/(Ra*T))*qsat)*Ra
    log.debug('numerator=%.3f'%num)
    den     = cpa + (cpc + (L/(Rc*T) - 1.)*(L/T))*qsat
    log.debug('denominator=%.3f'%den)
    log.debug('dlnT/dlnPa=%.3f'%(num/den))
    return num/den


# Dry adiabat function from p. 91 of RTP book as a comparison
def dry_adiabat( T_surf, P_array, cp_array ):

    T_dry   = np.zeros(len(P_array))
    P_surf  = np.amax(P_array)

    # Check if scalar or cp array
    if not isinstance(cp_array, list):
        cp_array = np.ones(len(P_array))*cp_array

    for idx, prs in enumerate(P_array):

        if cp_array[idx] > 0.:

            T_dry[idx] = T_surf * ( prs / P_surf ) ** ( phys.R_gas / cp_array[idx] )

    return T_dry

# Function for determining the dry adiabat pressure at a given temperature 
def dry_adiabat_pressure( P_surf, T_array, cp_array ):

    P_dry   = np.zeros(len(T_array))
    T_surf  = np.amax(T_array)

    # Check if scalar or cp array
    if not isinstance(cp_array, list):
        cp_array = np.ones(len(T_array))*cp_array

    for idx, tmp in enumerate(T_array):

        if cp_array[idx] > 0.:

            P_dry[idx] = P_surf * ( tmp / T_surf ) ** ( cp_array[idx] / phys.R_gas  )

    return P_dry

def dlnT_dlnP_d(lnP, lnT, atm):
    # T instead lnT
    tmp = math.exp(lnT)


    idx = int(np.amax(atm.ifatm))
    
    
    # Sum terms in equation
    num_sum     = 0.
    denom_sum  = 0. 

    
    # Calculate sums over volatiles
    for vol in atm.vol_list.keys(): 
        # Coefficients
        eta_vol     = atm.x_gas[vol][idx] / atm.xd[idx]
        eta_cond    = atm.x_cond[vol][idx] / atm.xd[idx]
        # sums for saturated comps
        # HII added, ensure below the critical point !
        if ((np.isclose(atm.p_vol[vol][idx] ,p_sat(vol,tmp, water_lookup=atm.water_lookup)) or
             atm.p_vol[vol][idx]  > p_sat(vol,tmp, water_lookup=atm.water_lookup)) and
            tmp < get_T_crit(vol)):

            beta = get_beta(vol, tmp, water_lookup=atm.water_lookup)
            L = L_heat(vol,tmp, water_lookup=atm.water_lookup)
            L_RT = L/phys.R_gas/tmp
            #beta_vol    = L_heat(vol, tmp) / (R_gas * tmp) #RTP
            # Sum in numerator
            num_sum     += eta_vol * L_RT*phys.R_gas
            # Sum in denominator
            denom_sum += eta_vol*(cpv(vol,tmp)  + phys.R_gas*beta*(L_RT - 1) +
                                  atm.alpha_cloud * eta_cond * cp_cond(vol,tmp))
            
        # sums for subsaturated comps
        else: 
            denom_sum += cpv(vol,tmp) * eta_vol #The eta_vol (x_vol/x_d) is there so that if there are multiple dry components the average dry specific heat is used
        
    
        
    
    # Collect terms
    numerator   = phys.R_gas + num_sum
    denominator = denom_sum

    # dlnT/dlnPd
    dlnTdlnPd = numerator / denominator

    # Moist adiabat slope
    return dlnTdlnPd


# In[70]:


def moist_slope(lnP, lnT, atm):
    # T instead lnT
    tmp = math.exp(lnT)
    
    idx = int(np.amax(atm.ifatm))
    # Sum terms in equation
    num_sum     = 0.
    denom_sum  = 0. 

    
    
    # Calculate sums over volatiles
    for vol in atm.vol_list.keys(): 
        
        # Coefficients
        eta_vol     = atm.x_gas[vol][idx] / atm.xd[idx]
        
        # sums for saturated comps
        # Ensure below the critical point for volatile before we
        # assume condensation occurs
        if ((np.isclose(atm.p_vol[vol][idx] ,p_sat(vol,tmp,water_lookup=atm.water_lookup)) or
             atm.p_vol[vol][idx]  > p_sat(vol,tmp,water_lookup=atm.water_lookup)) and
            tmp < get_T_crit(vol)):
            
            # Sum in numerator
            num_sum     += eta_vol
            
            # Sum in denominator
            beta = get_beta(vol, tmp, water_lookup=atm.water_lookup)
            denom_sum  += beta * eta_vol * dlnT_dlnP_d(lnP,lnT,atm)
             
    
    # Collect terms
    numerator   = 1 + num_sum
    denominator = 1 + denom_sum
    dlnPd_dlnP = numerator / denominator
    # dlnT/dlnPd
    dlnTdlnP = dlnPd_dlnP * dlnT_dlnP_d(lnP,lnT,atm)

    # Moist adiabat slope
    return dlnTdlnP




def condensation( atm, idx, wet_list, dry_list, prs_reset):

    # Temperature floor
    tmp = np.amax([atm.tmp[idx], atm.minT])
    
    
    if idx==0:
        
        # Renormalize mixing ratios to ensure sum == 1
        if sum(atm.vol_list.values()) != 1.:
            vol_list_new = {}
            for vol in atm.vol_list.keys():
                vol_list_new[vol]   = atm.vol_list[vol] / sum(atm.vol_list.values())
            for vol in atm.vol_list.keys():
                atm.vol_list[vol] = vol_list_new[vol]
        # Partial pressures scaled 
        for vol in atm.vol_list:
            atm.p_vol[vol][idx] = atm.vol_list[vol] * atm.p[idx]
            #atm.p_vol[vol][idx] = atm.vol_list[vol] * p_tot_pre_condensation
            # Mean gas phase molar mass
            atm.mu[idx]       += atm.vol_list[vol] * phys.molar_mass[vol]
    
    else:  
        
        # Total pressure & partial pressures of condensing phases
        p_cond_sum = 0
        for vol in atm.vol_list:
            if vol in wet_list:
                atm.p_vol[vol][idx] = p_sat(vol, atm.tmp[idx],water_lookup=atm.water_lookup)
                p_cond_sum += atm.p_vol[vol][idx]
                
        # Calculate the total partial pressure of dry species
        p_dry_tot = atm.p[idx] - p_cond_sum
        dry_frac_sum = 0
        # calculate sum of fractions of dry species for partial pressure calculation
        for vol in atm.vol_list:
            if vol in dry_list:
                dry_frac_sum += atm.x_gas[vol][idx-1]
        # Calculate the individual partial pressures of dry species
        for vol in atm.vol_list:
            if vol in dry_list:
                atm.p_vol[vol][idx] = p_dry_tot * (atm.x_gas[vol][idx-1]/dry_frac_sum)
                
       
    # Calculate mean molar mass
    atm.mu[idx]   = 0.
    
    for vol in atm.vol_list.keys():
        atm.mu[idx]   += phys.molar_mass[vol] * atm.p_vol[vol][idx]
    atm.mu[idx] /= atm.p[idx]
        
    

    
    # Update condensate mixing ratios
    for vol in atm.vol_list.keys():
       
        # Condensate phase
        if atm.p_vol[vol][idx] < p_sat(vol, tmp,water_lookup=atm.water_lookup):
            atm.x_cond[vol][idx] = 0.
        else:
            #atm.x_cond[vol][idx] = atm.vol_list[vol] - ( atm.p_vol[vol][idx] / atm.p[idx] )
            if idx == 0:
                #atm.x_cond[vol][idx] = atm.vol_list[vol] - ( mu_old / atm.mu[idx] * atm.p_vol[vol][idx] / p_tot_pre_condensation )
                atm.x_cond[vol][idx] = 0.
            else:
                #mu_old = atm.mu[idx-1]
                atm.x_cond[vol][idx] = atm.x_cond[vol][idx-1] + atm.x_gas[vol][idx-1] - (1-atm.xc[idx-1])*(atm.p_vol[vol][idx]/atm.p[idx])

        #atm.x_cond[vol][idx] *= atm.alpha_cloud

        # Add to molar concentration of total condensed phase
        atm.xc[idx]     += atm.x_cond[vol][idx]
    for vol in atm.vol_list.keys():    
        # Gas phase molar concentration
        #atm.x_gas[vol][idx] = atm.p_vol[vol][idx] / atm.p[idx]
        if idx == 0:
            #atm.x_gas[vol][idx] =  mu_old / atm.mu[idx] * atm.p_vol[vol][idx] /  p_tot_pre_condensation
            atm.x_gas[vol][idx] = atm.vol_list[vol]
            
        else:
            atm.x_gas[vol][idx] =  ( 1-atm.xc[idx] ) * atm.p_vol[vol][idx] /  atm.p[idx]
            #atm.x_gas[vol][idx] =  mu_old / atm.mu[idx] * atm.p_vol[vol][idx] /  p_tot_pre_condensation
        
        # Add to molar concentration of total gas (dry or moist) phase
        # ! REVISIT ! keeping xd == 0 leads to a bug, why?
        if vol in dry_list:
            atm.xd[idx]          += atm.x_gas[vol][idx]
        if vol in wet_list:
            atm.xv[idx]          += atm.x_gas[vol][idx]
        
        # Mean cp of both gas phase and retained condensates
        atm.cp[idx]    += atm.x_gas[vol][idx] * cpv(vol, atm.tmp[idx]) + atm.x_cond[vol][idx] * cp_cond(vol, atm.tmp[idx]) * atm.alpha_cloud

    # Renormalize cp w/ molar concentration (= cp_hat, Eq. 1, Li+2018)
    atm.cp[idx]  = atm.cp[idx] / ( atm.xd[idx] + atm.xv[idx] )

    # Dry concentration floor
    atm.xd[idx]  = np.amax([atm.xd[idx], 1e-10])
    
    # Loop through species to determine wet_list and dry_list for next level
    
    for vol in atm.vol_list.keys():
        
        
        # Condensation if p_i > p_sat
        if atm.p_vol[vol][idx] >= p_sat(vol, tmp,water_lookup=atm.water_lookup):
            
            # Add condensing species to wet_list
            if vol not in wet_list:
                wet_list.append(vol)
                
            
            # Set species partial pressure to p_sat
            #atm.p_vol[vol][idx]  = p_sat(vol, tmp)
            
            # Add the species partial pressure to the condensing species sum
            #p_cond_sum += atm.p_vol[vol][idx]
            
            # If the species is in dry_list from previous iteration of while loop, remove it
            if vol in dry_list:
                
                dry_list.remove(vol)
            
        else:
            
            # Add a non-condensing species to the list of dry species
            # if it's present (>0) and if it's not in a list already
            if atm.vol_list[vol] > 0. and vol not in dry_list and vol not in wet_list:
                dry_list.append(vol)
    
    return atm, wet_list, dry_list

# Builds the generalized moist adiabat from slope(lnP, lnT, atm object)
def general_adiabat( atm ):
    # First, we check to see if the initial vol_list/Tsurf/Psurf  
    # combo gives a super-saturated surface state. If so, we remove material
    # until the super-saturated phase is just saturated; none of this condensate
    # is retained because the initial supersaturated state is unrealistic.
    # Adjust the partial pressures of the supersaturated species to their
    # saturation vapor pressures, then adjust Vol_list and Psurf.
    new_psurf = 0
    new_p_vol = {}
    wet_list = []
    dry_list = []
    for vol in atm.vol_list.keys():
        if atm.vol_list[vol] * atm.ps > p_sat(vol, atm.ts,water_lookup=atm.water_lookup):
            new_psurf += p_sat(vol,atm.ts,water_lookup=atm.water_lookup)
            new_p_vol[vol] = p_sat(vol,atm.ts,water_lookup=atm.water_lookup)
        else:
            new_psurf += atm.vol_list[vol] * atm.ps
            new_p_vol[vol] = atm.vol_list[vol] * atm.ps
            
    if new_psurf != atm.ps:
        # Backup variables before they are lost
        Tsurf = atm.ts
        band_edges = atm.band_edges
        minT = atm.minT
        maxT = atm.maxT
        nlev_save = atm.nlev_save
        re=atm.effective_radius
        lwm=atm.liquid_water_fraction
        clfr=atm.cloud_fraction
        do_cloud=atm.do_cloud
        alpha_cloud=atm.alpha_cloud

        attrs = {}
        for a in ["instellation", "zenith_angle", "albedo_pl", 
                    "inst_sf", "skin_k", "skin_d", "tmp_magma", "albedo_s",
                    "planet_mass", "planet_radius"]:
            attrs[a] = getattr(atm,a)

        # Calc new mixing ratios
        new_vol_list = {}
        for vol in atm.vol_list.keys():
           new_vol_list[vol] = new_p_vol[vol] / new_psurf

        # New atmos object
        atm = atmos(Tsurf, new_psurf, atm.ptop, atm.planet_radius, atm.planet_mass, band_edges,
                    vol_mixing=new_vol_list, trppT=atm.trppT, minT=minT, maxT=maxT, req_levels=nlev_save,
                    re=re, lwm=lwm, clfr=clfr, do_cloud=do_cloud, alpha_cloud=alpha_cloud
                    )

        # Restore backed-up variables
        for a in attrs.keys():
            setattr(atm,a,attrs[a])

    for vol in atm.vol_list.keys():
        if np.isclose(atm.vol_list[vol] * atm.ps, p_sat(vol,atm.ts,water_lookup=atm.water_lookup)):
            wet_list.append(vol)
        elif atm.vol_list[vol] > 0:
            dry_list.append(vol)
    
    ### Initialization
    
    # Initialize the tuple solution
    moist_tuple     = [] #[tuple([np.log(atm.ps), atm.ts])] 
   
    # Negative increment to go from ps to ptop < ps       
    step            = -.01

    # Integration counter
    idx             = 0  
    
    # Surface condensation
    atm, wet_list, dry_list  = condensation(atm, idx, wet_list, dry_list, prs_reset=False)

    # Create the integrator instance                                              
    int_slope       = integrator(moist_slope, np.log(atm.ps), np.log(atm.ts), step)

    # Update parameters used in the slope function dlntT/dlnP
    int_slope.setParams(atm)
    
    ### Integration of full general adiabat
    while atm.p[idx] > atm.ptop:
        
        # Execute the Runge-Kutta integrator, fill array of tuples
        moist_tuple.append(int_slope.next())

        # Fill next T,P values
        atm.p[idx+1]    = np.exp(int_slope.x)
        atm.tmp[idx+1]  = np.exp(int_slope.y)

        # Calculate next local gravity and height
        atm = atm_z(atm, idx)

        # Set next level to calculate
        idx             += 1
        atm.ifatm[idx]  = idx

        # Calculate condensation at next level
        atm, wet_list, dry_list    = condensation(atm, idx, wet_list, dry_list, prs_reset=False)

    # Interpolate
    atm = interpolate_atm(atm)
    
    # Rescale mixing ratios for plotting purposes
    atm.xd[:] *= 1 / ( 1 - (1-atm.alpha_cloud) * atm.xc[:])
    atm.xv[:] *= 1 / ( 1 - (1-atm.alpha_cloud) * atm.xc[:])
    for vol in atm.vol_list.keys():
        if atm.vol_list[vol] > 1e-10:
            atm.x_cond[vol][:] *= atm.alpha_cloud / ( 1 - (1-atm.alpha_cloud) * atm.xc[:])
            atm.x_gas[vol][:] *= 1 / ( 1 - (1-atm.alpha_cloud) * atm.xc[:])
    atm.xc[:] *= atm.alpha_cloud / ( 1 - (1-atm.alpha_cloud) * atm.xc[:])
    
    
    return atm

# Interpolate and flip pressure, temperature and volatile grids to fixed size
def interpolate_atm(atm):

    # Trim array zeros
    atm_len     = int(np.max(atm.ifatm)+1)
    rest_len    = int(len(atm.p)-atm_len)
    atm.p       = np.flip(np.split(atm.p, [atm_len, rest_len])[0], axis=0)
    atm.tmp     = np.flip(np.split(atm.tmp, [atm_len, rest_len])[0], axis=0)

    # Interpolate staggered nodes
    atm.pl      = np.logspace(np.log10(np.min(atm.p)), np.log10(np.max(atm.p)), atm.nlev_save+1)
    atm.tmpl    = np.interp(atm.pl, atm.p, atm.tmp)

    # Interpolate atmosphere nodes
    prs_itp     = (atm.pl[1:] + atm.pl[:-1]) / 2.
    tmp_itp     = (atm.tmpl[1:] + atm.tmpl[:-1]) / 2.

    # Trim level-dependent quantities
    atm.grav_z  = np.flip(np.split(atm.grav_z, [atm_len,rest_len])[0], axis=0)
    atm.z       = np.flip(np.split(atm.z, [atm_len,rest_len])[0], axis=0)
    atm.mu      = np.flip(np.split(atm.mu, [atm_len,rest_len])[0], axis=0)
    atm.xd      = np.flip(np.split(atm.xd, [atm_len, rest_len])[0], axis=0)
    atm.xv      = np.flip(np.split(atm.xv, [atm_len, rest_len])[0], axis=0)
    atm.xc      = np.flip(np.split(atm.xc, [atm_len, rest_len])[0], axis=0)
    atm.cp      = np.flip(np.split(atm.cp, [atm_len, rest_len])[0], axis=0)
    atm.rho     = np.flip(np.split(atm.rho, [atm_len, rest_len])[0], axis=0)
    # Interpolate level-dependent quantities
    atm.grav_z  = np.interp(prs_itp, atm.p, atm.grav_z)
    atm.z       = np.interp(prs_itp, atm.p, atm.z)
    atm.mu      = np.interp(prs_itp, atm.p, atm.mu)
    atm.xd      = np.interp(prs_itp, atm.p, atm.xd)
    atm.xv      = np.interp(prs_itp, atm.p, atm.xv)
    atm.xc      = np.interp(prs_itp, atm.p, atm.xc)
    atm.cp      = np.interp(prs_itp, atm.p, atm.cp)
    atm.rho     = np.interp(prs_itp, atm.p, atm.rho)
    # Trim & interpolate species-dependent quantities
    for vol in atm.vol_list.keys():
        
        # Trim
        atm.p_vol[vol]   = np.flip(np.split(atm.p_vol[vol], [atm_len, rest_len])[0], axis=0)
        atm.x_gas[vol]   = np.flip(np.split(atm.x_gas[vol], [atm_len, rest_len])[0], axis=0)
        atm.x_cond[vol]  = np.flip(np.split(atm.x_cond[vol], [atm_len, rest_len])[0], axis=0)
        
        # Interpolate
        atm.pl_vol[vol]  = np.interp(atm.pl, atm.p, atm.p_vol[vol]) # staggered!
        atm.p_vol[vol]   = np.interp(prs_itp, atm.p, atm.p_vol[vol])
        atm.x_gas[vol]   = np.interp(prs_itp, atm.p, atm.x_gas[vol]) 
        atm.x_cond[vol]  = np.interp(prs_itp, atm.p, atm.x_cond[vol])

    # Rewrite atmosphere nodes
    atm.p       = prs_itp
    atm.tmp     = tmp_itp

    return atm


def plot_adiabats(atm,filename='output/general_adiabat.pdf'):

    # sns.set_style("ticks")
    # sns.despine()

    ls_moist    = 2.5
    ls_dry      = 2.0
    ls_ind      = 1.5

    fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(13,6))#, sharey=True)
    # sns.set_style("ticks")
    # sns.despine()
    
    
    # For reference p_sat lines
    T_sat_array    = np.linspace(20,3000,1000) 
    p_partial_sum  = np.zeros(len(atm.tmp))
    
    vol_list_sorted = {k: v for k, v in sorted(atm.vol_list.items(), key=lambda item: item[1])}

    # Individual species
    for vol in vol_list_sorted.keys():

        # Only if volatile is present
        if atm.vol_list[vol] > 1e-10:
    
            # Saturation vapor pressure for given temperature
            Psat_array = [ p_sat(vol, T,water_lookup=atm.water_lookup) for T in T_sat_array ]
            ax1.semilogy( T_sat_array, Psat_array, label=r'$p_\mathrm{sat}$'+vol_latex[vol], lw=ls_ind, ls=":", color=vol_colors[vol])

            # Plot partial pressures
            ax1.semilogy(atm.tmp, atm.p_vol[vol], color=vol_colors[vol], lw=ls_ind, ls="-", label=r'$p$'+vol_latex[vol],alpha=0.99)

            # Sum up partial pressures
            p_partial_sum += atm.p_vol[vol]

            # Plot individual molar concentrations
            ax2.semilogy(atm.x_cond[vol],atm.p, color=vol_colors[vol], lw=ls_ind, ls="--", label=vol_latex[vol]+" cond.")
            ax2.semilogy(atm.x_gas[vol],atm.p, color=vol_colors[vol], lw=ls_ind, ls="-", label=vol_latex[vol]+" gas")
            
    # # Plot sum of partial pressures as check
    ax1.semilogy(atm.tmp, p_partial_sum, color="green", lw=ls_dry, ls="--", label=r'$\sum p^\mathrm{i}$',alpha=0.99)
    
    # # # Dry adiabat function from RTB book
    # # ! careful: non-T dependent CPs used, can lead to differences with general adiabat !
    ax1.semilogy( dry_adiabat( atm.ts, atm.pl, atm.cp[-1]), atm.pl , color=vol_colors["black_3"], ls="-.", lw=ls_dry, label=r'Dry adiabat') # Functional form

    # General moist adiabat
    ax1.semilogy(atm.tmpl, atm.pl, color=vol_colors["black_1"], lw=ls_moist,label="General adiabat",alpha=0.99)

    # Phase molar concentrations
    ax2.semilogy(atm.xd+atm.xv,atm.p, color=vol_colors["black_2"], lw=ls_ind, ls=":", label=r"Gas phase")

    fs_l = 16
    fs_m = 14
    fs_s = 12

    ax1.invert_yaxis()
    ax1.set_xlabel(r'Temperature, $T$ (K)', fontsize=fs_l)
    ax1.set_ylabel(r'Pressure, $P$ (Pa)', fontsize=fs_l)
    # ax1.set_title('Adiabats & individual Clausius-Clapeyron slopes', fontsize=fs_l)
    ax1.legend(loc=1, ncol=np.min([len(atm.vol_list)+1,2]), fontsize=fs_s)
    ax1.set_xlim([0,np.max(atm.ts)])

    ax2.invert_yaxis()
    # ax2.set_title('Phase & species abundances', fontsize=fs_l)
    ax2.set_xlabel(r'Molar concentration, $X^{\mathrm{i}}_{\mathrm{phase}}$', fontsize=fs_l)
    ax2.set_ylabel(r'Pressure, $P$ (Pa)', fontsize=fs_l)
    ax2.legend(loc=2, ncol=2, fontsize=fs_s)

    ax1.set_ylim(top=atm.ptop)
    ax1.set_ylim(bottom=atm.ps)
    ax2.set_ylim(top=atm.ptop)
    ax2.set_ylim(bottom=atm.ps)

    ax2.set_xscale("log")
    ax2.set_xlim([1e-4, 1.05])
    ax2.set_xticks([1e-4, 1e-3, 1e-2, 1e-1, 1e0])
    ax2.set_xticklabels(["$10^{-4}$", "0.001", "0.01", "0.1", "1"])
    # ax2.set_xlim(right=1.1)

    ax1.tick_params(axis='both', which='major', labelsize=fs_m)
    ax1.tick_params(axis='both', which='minor', labelsize=fs_m)
    ax2.tick_params(axis='both', which='major', labelsize=fs_m)
    ax2.tick_params(axis='both', which='minor', labelsize=fs_m)
    
    ax1.text(0.02, 0.015, 'A', color="k", rotation=0, ha="left", va="bottom", fontsize=fs_l+3, transform=ax1.transAxes)
    ax2.text(0.02, 0.015, 'B', color="k", rotation=0, ha="left", va="bottom", fontsize=fs_l+3, transform=ax2.transAxes)
    #fig.suptitle(r'$\alpha$=%.1f'%atm.alpha_cloud)
    fig.suptitle('$T_{surf}$=%.1f K, $T_{bot}$=%.1f K'% (atm.ts,atm.tmp[-1]))
    #plt.show()

    fig.savefig(filename, bbox_inches='tight', dpi=190)
    plt.close()

    return


####################################
##### Stand-alone initial conditions
####################################
if __name__ == "__main__":

    # Surface temperature & partial pressures
    T_surf                  = 400                           # K
    pH2O                    = p_sat('H2O',T_surf)           # Pa
    pCO2                    = 0.                            # Pa
    pH2                     = 0.                            # Pa
    pN2                     = 3e+5                          # Pa
    pCH4                    = 0.                            # Pa
    pO2                     = 0.                            # Pa
    pO3                     = 0.                            # Pa
    pHe                     = 0.                            # Pa
    pNH3                    = 0.                            # Pa
    P_surf                  = pH2O + pCO2 + pH2 + pN2 + pCH4 + pO2 + pO3 + pHe + pNH3  # Pa

    # Set fraction of condensate retained in column (0 = full rainout)
    alpha_cloud             = 0.0

    pl_radius     = 6.371e6             # m, planet radius
    pl_mass       = 5.972e24            # kg, planet mass
    P_top         = 1.0                 # Pa

    # Cloud parameters
    re   = 1.0e-5 
    lwm  = 0.8   
    clfr = 0.0    

    # Volatile molar concentrations in the dictionary below are defined as fractions that must sum to one
    # The vanilla setting defines a water-saturated atmosphere with a 3 bar N2 background
    vol_list = { 
                  "H2O" : pH2O / P_surf,   
                  "CO2" : pCO2 / P_surf,
                  "H2"  : pH2  / P_surf,
                  "N2"  : pN2  / P_surf,
                  "CH4" : pCH4 / P_surf,
                  "O2"  : pO2  / P_surf,
                  "O3"  : pO3  / P_surf,
                  "CO"  : pN2  / P_surf,
                  "He"  : pHe  / P_surf,
                  "NH3" : pNH3 / P_surf,
                }
    # Create atmosphere object
    atm                     = atmos(T_surf, P_surf, P_top, pl_radius, pl_mass, re, lwm, clfr, [], vol_mixing=vol_list)

    # Set fraction of condensate retained in column (0 = full rainout)
    atm.alpha_cloud         = alpha_cloud
    
    # Calculate moist adiabat + condensation
    atm                     = general_adiabat(atm)

    # Plot adiabat
    plot_adiabats(atm, filename="../output/general_adiabat.pdf")
    



#----------Section 3: Plotting utilities-------------------------------
#Graphics interface customized for MatPlotLib
#
#This is imported into ClimateUtilities. If you want to use a
#different graphics package (e.g. MatPlotLib) as a graphics driver
#in place of Ngl, you only need to rewrite this module. The most
#important thing to rewrite is the plot(...) function, which takes
#a Curve object as input and produces a line plot.  The plot(...)
#function returns a plotObj object to allow further manipulation
#(mostly saving the plot in various formats), but if you have a plotting
#package that provides other ways of saving the plots, or if you just
#want to save the plots by doing screen dumps, you can just have
#the plot(...) function return None, or some dummy object.
#
#The other main function to customize is contour(...) which produces
#contour plots of an array. This function is not very extensively
#used in the courseware, so it is a lower priority for modification.
#This routine has not yet been implemented for MatPlotLib
#
#ToDo:
#    *There is a deprecation warning relating to handling of axis
#     instances.  Figure out what is going on and take care of it.
#-----------------------------------------------------------------------

#Change Log:
#     1/25/2012: Fixed bug in legends that made problem with Enthought 7.2
#
#     11/12/2013: plot()--Implemented plotting of multiple curves with
#                 differing resolution (MPL only, not NGL yet)

#Try to import matplotlib plotting routines 
try:
    import matplotlib as mpl
    #Configure the backend so things work properly
    #with the Python intepreter and idle. Not needed if you are
    #using ipython. To work properly with idle, idle needs
    #to be started up with the command "idle -n".  Note that
    #this trick limits the options for saving a file; only png
    #format is supported. If you want higher resolution formats
    #(notably eps) you should run using ipython -pylab , and eliminate
    #the following two commands.
    mpl.rcParams['backend'] = 'TkAgg'
    mpl.rcParams['interactive'] = True
    #
    import pylab as pl
except:
    print('matplotlib not found on your system')

#A dummy class useful for passing parameters.
#Just make an instance, then add new members
#at will. Not sure why it also has to be defined
#here, since it is also defined in ClimateUtilities,
#but it seems to be necessary
class Dummy:
    pass

# A little class to make resources for Ngl plot options
class resource:
    def __init__(self):
        self.trYReverse = False
        self.trXReverse = False
        self.trXLog = False
        self.trYLog = False
        self.nglFrame = False
        self.nglDraw = False
        #ToDo: Missing data code resource, line styles, line colors

# A little class for use as a return object from plot(), so
# the user has an easy way to delete a window or save a plot
# to an eps file. Plot objects not implemented yet for MPL
# Is there a way to save plots non-interactively in MPL?
class plotObj:
    def __init__(self,workstation,plot,WorkstationResources=None):
        self.workstation = workstation
        self.plot = plot
        self.WorkstationResources = WorkstationResources
    #Deletes a plot window
    def delete(self):
        print("In MatPlotLib just click the goaway button to dispose a plot")
    def save(self,filename = 'plot'):
        #**ToDo: Can we implement non-interactive save in MPL?
        print("In MatPlotLib save plots interactively using the file button")
        print(" in the plot window")
        
#ToDo:
#       *Implement use of missing data coding.
#       *Provide some way to re-use window (e.g. by
#         returning w, and having it be an optional
#         argument. How to clear a window for re-use?
#       *Make some use of dashed lines in line styles
#       *The behavior of axis options like reverseX and
#        XlogAxis is confusing when switchXY = True. We
#        need to think about the semantics of what we
#        mean by the "X" axis, and stick to it.  As
#        currently implemented the "X" axis means the
#        horizontal axis for these options.  (However,
#        the semantics is different for the labeling options!)
#        If we change this (and we probably should), the
#        scripts plotting soundings will also need to be changed.
#        as well as the Workbook intructions and problem sets.
#

#List of line colors and line styles to use
lineColors = ['b','g','r','c','m','y','k']
lineStyles = ['-','--','-.',':'] 
lineThickness = [2,3,4]
plotSymbols = ['.','o','v','<','s','*','+','x']
def plot(*curves):
    '''
    Plots Curve objects. plot(...) takes any number of Curve objects
    as arguments, and plots all of them on a single graph.  If multiple
    Curve objects are passed as arguments, the plot appearance data
    (axis options, title, axis labels, etc) are taken from the first
    Curve object. The Curve objects need not all have the same data length.
    This is convenient for plotting data from multiple sources, with varying
    resolution. 
    '''
    cMaster = curves[0] #Appearance options are taken from this Curve
    
    fig = pl.figure() #Always start a new figure
    #**ToDo: Make sure log-axis options so they work consistently
    #        with Ngl implementation when x/y axes are switched.
    #        (Looks OK, but is that implementation the logical way?)
    #r.trXLog = c.XlogAxis
    #r.trYLog = c.YlogAxis
    if cMaster.XlogAxis:
        pl.semilogx()
    if cMaster.YlogAxis:
        pl.semilogy()
    if cMaster.XlogAxis & cMaster.YlogAxis:
        pl.loglog()
    #
    #Line styles (Not needed in MPL, handled automatically)
    #r.xyLineColors = lineColors
    #r.xyLineThicknesses = lineThickness
    #Plot title
    #r.tiMainString = c.PlotTitle
    pl.title(cMaster.PlotTitle)
    #Axis labels (ToDo: add defaults)
    #X and Y axis labels    
    #r.tiXAxisString = c.Xlabel
    #r.tiYAxisString = c.Ylabel
    if cMaster.switchXY:
        pl.ylabel(cMaster.Xlabel)
        pl.xlabel(cMaster.Ylabel)
    else:
        pl.xlabel(cMaster.Xlabel)
        pl.ylabel(cMaster.Ylabel)
        
    #  Legends, for multicurve plot
    legends = []
    for c in curves:
        for id in c.listVariables():
            if not id == c.Xid:
                if len(c.label[id]) > 0:
                    legends.append(c.label[id])
                else:
                    legends.append(id)
    #ToDo: Add option to skip legends
    #
    #Suppress line drawing and just plot symbol for scatter plot curves
    #**ToDo: Implement for MatPlotLib
    #r.xyMarkers = plotSymbols
    #r.xyMarkerColors = lineColors
    #r.xyMarkerSizeF = .01
    #r.xyMarkLineModes = []
    formatList = []
    count = 0
    for c in curves:
        for id in c.listVariables():
            if not id == c.Xid:
                if c.scatter[id]:
                    #r.xyMarkLineModes.append('Markers')
                    color = lineColors[count%len(lineColors)]
                    symbol = plotSymbols[count%len(plotSymbols)]
                    formatList.append(color+symbol) 
                else:
                    color = lineColors[count%len(lineColors)]
                    style = lineStyles[count%len(lineStyles)]
                    formatList.append(color+style)
                count += 1
    #
    plotList = [] #Mainly so we can add legends. Could be done with label = ...
    #Note that pl.plot returns a list of lines, even if there is only
    #     one line in the plot, so for legends to work correctly,
    #     we need to extract just the first element.
    countAll = 0
    for c in curves:
        if cMaster.switchXY:
            count = 0
            for data in c.Y():
                plotList.append(pl.plot(data,c.X(),formatList[countAll])[0])
                count += 1
                countAll += 1
        else:
            count = 0
            for data in c.Y():
                plotList.append(pl.plot(c.X(),data,formatList[countAll])[0])
                count += 1
                countAll += 1
    #Do the legends
    #11/12/2013: Added Jonah's trick to put legends on the side
    ax = pl.subplot(111)
    box = ax.get_position()
    ax.set_position([box.x0, box.y0, box.width * 0.8, box.height])
    pl.legend(plotList,legends,loc='center left', bbox_to_anchor=(1, 0.5))
    #
    #Do the axis reversal
    #We do it here, since we don't know the axis limits until
    #plotting is done
    #r.trYReverse = c.reverseY
    #r.trXReverse = c.reverseX
    axes = pl.gca() #Gets current axis
    if cMaster.reverseX:
        axes.set_xlim(axes.get_xlim()[::-1]) #::-1 reverses the array
    if cMaster.reverseY:
        axes.set_ylim(axes.get_ylim()[::-1])
    #Now re-draw the plot
    pl.draw()
    #(Insert commands needed to show plot, if necessary)
    return plotObj(None,fig) #Eventually we will use this to make subplots and do save option

# A basic contour plotter, which will plot a contour plot
# of a numpy array. The x and y scales can optionally
# be specified using keyword arguments x and y. For example,
# if we want the x scale to be the array (or list) lat, and
# the y scale to be the array (or list) lon, we would call
# contour as contour(A,x=lat,y=lon).
def contour(A,**kwargs):
    #**ToDo: Add labeled contour lines, option to change contour levels,
    #        and to change palette.
    #
    fig = pl.figure() #Always start a new figure
    # Set axes if they have been specified
    # as keyword arguments
    if 'x' in kwargs.keys():
        x = kwargs['x']
    else:
        x = range(A.shape[1])
    if 'y' in kwargs.keys():
        y = kwargs['y']
    else:
        y = range(A.shape[0])
    cs = pl.contourf(x,y,A)
    cbar = pl.colorbar(cs)
    return plotObj(None,fig)
##    #The following allows an expert user to pass
##    #Ngl options directly to the plotter.
##    #ToDo: Note that
##    #options explicitly specified later will over-ride
##    #what the user specified. This should be fixed,
##    #by checking for things already specified. We should
##    #also allow for using this resource to specify a color
##    #map.
##    if 'resource' in kwargs.keys():
##        r = kwargs['resource']
##    else:
##        r = Dummy()
##    
##    rw = Dummy()
##    #Set the color map
##    if 'colors' in kwargs.keys():
##        if (kwargs['colors'] == 'gray') or (kwargs['colors'] == 'grey') :
##            #Set the default greyscale
##            rw.wkColorMap = 'gsdtol'
##        else:
##            rw.wkColorMap = kwargs['colors']
##    else:
##        #Default rainbow color table
##        rw.wkColorMap = "temp1" 



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jan 23 12:10:01 2023

@authors:    
Ryan Boukrouche (RB)
Harrison Nicholls (HN)
"""

import numpy as np
import janus.utils.GeneralAdiabat as ga # Moist adiabat with multiple condensibles

def moist_adj(atm, conv_timescale, nb_convsteps = 10):
    """Moist adjustment routine, as formulated by RB.

    Assumes constant latent heat and does not conserve moist enthalpy. 

    Parameters
    ----------
        atm : atmos
            Atmosphere object
        conv_timescale : float
            Convective timescale

        nb_convsteps : int
            Number of passes

    Returns
    ----------
        dT_conv : np.ndarray
            Array of temperature changes introduced by adjustment

    """

    nlev = len(atm.p)
    
    dT_conv = np.zeros(nlev)
    Tmid_cc = atm.tmp.copy()

    if not ("H2O" in atm.vol_list.keys()):
        return dT_conv
        
    for _ in range(nb_convsteps):
        did_adj = False
        #------------------- L = cst -------------------
        for i in range(nlev-1): #Downward pass
            pp_h2o = atm.p_vol["H2O"][i]
            if (pp_h2o < 1e-10):
                continue
            if (Tmid_cc[i] < ga.Tdew('H2O',pp_h2o)):
                Tmid_cc[i]=ga.Tdew('H2O',pp_h2o)
                did_adj = True

        for i in range(-2,1,-1): #Upward pass
            pp_h2o = atm.p_vol["H2O"][i]
            if (pp_h2o < 1e-10):
                continue
            if (Tmid_cc[i] < ga.Tdew('H2O',pp_h2o)):
                Tmid_cc[i]=ga.Tdew('H2O',pp_h2o)
                did_adj = True

        # If no adjustment required, exit the loop
        if (did_adj == False):
            break

    # Change in temperature is Tmid_cc - Tmid
    dT_conv[:] = (Tmid_cc[:] - atm.tmp[:])/conv_timescale
    return dT_conv



'''
GGRadConv.py
'''

import math
import numpy as np
import matplotlib.pyplot as plt

import janus.utils.phys as phys
import janus.utils.planets as planets
import janus.utils.GreyHeat as Grey


#Set the gravity and thermodynamic constants
Rcp = 2./7.
n=20


#Choose the radiation model for computing the longwave and shortwave heating
radcomp = Grey.radcomp

def RadConvEqm(Tg):
    #--------------------Set radmodel options-------------------
    #---Instantiate the radiation model---
    n = 20
    #

    #Set global constants
    ps = 1000.
    rh = 1.e-30#Relative humidity
    rhbdd = 1.e-30
    dt = 24.*3600. #time step in seconds

    #---Set up pressure array (a global)----
    ptop = 50. #Top pressure in mb (Changed from 1mb in original)
    pstart = .995*ps
    rat = (ptop/pstart)**(1./n)
    logLevels = [pstart*rat**i for i in range(n)]
    logLevels.reverse()
    levels = [ptop + i*(pstart-ptop)/(n-1) for i in range(n)]
    p = numpy.array(logLevels)



    #==============Now do the calculation====================================

    #--------------Initializations-------------------------------------------

    #----------------Set initial time step--------------------------------

    dtime = 1.# 1. # (for CO2 case; gray gas evolves faster)

    #----------------------------------------------------------------------

    #---Temperature and moisture arrays (initialized)
    T = numpy.zeros(n) + 230.


    #--------------Other parameters-------------------------------------------
    doStellarAbs = False

    #Set composition constants (globals)
    #co2 = 300.

    #Ground temperature (held fixed in this computation)
    Tg = 280.
    #---Temperature and moisture arrays (initialized)
    T = Tg*(p/p[-1])**Rcp  #Initialize on an adiabat
    #T = Tg*numpy.ones(len(p))


    #Set composition parameters for the radiation code you are using
    q=np.zeros(n)


    #Grey Gas:
    Grey.tauInf = 2.

    PrevOLR = 0.
    #---------------------------------------------------------
    #--------------Initializations Done-----------------------
    #--------------Now do the time stepping-------------------
    #---------------------------------------------------------
    for i in range(0,50):
        nout = 10*i
        print(dtime)
        #if i%50 == 0 & i > 200:
        #     dtime = .5*dtime
        Tg,Tad,T,flux,fluxStellar,fluxLW,heat,heatStellar,heatLW = steps(Tg,T,p,q,10,dtime)
        print('History step',Tg,flux[-1],max(heat),min(heat))
        #history(nout,caseTag)
        if abs(flux[-1]-PrevOLR) < 1.0:
               break    # break here
        PrevOLR = flux[-1]

    # plot equilibrium temperature profile
    plt.figure()
    plt.semilogy(T,p)
    plt.gca().invert_yaxis()
    plt.ylabel('Pressure (mb)')
    plt.xlabel('Temperature (K)')
    plt.savefig('Tprofile.pdf',bb_inches='tight')


    return flux[-1]



#Dry adjustment routine.
#**ToDo: Modify so it handles non-uniform pressure levels
#        correctly, and conserves integrated enthalpy cp T
#
#Iterative routine for dry convective adjustment
def dryAdj(T,p):
    #Rcp is a global
    #Downward pass
    for i in range(len(T)-1):
        T1,p1 = T[i],p[i]
        T2,p2 = T[i+1],p[i+1]
        pfact = (p1/p2)**Rcp
        if T1 < T2*pfact:
            Tbar = .5*(T1+T2) #Equal layer masses
                              #Not quite compatible with how
                              #heating is computed from flux
            T2 = 2.*Tbar/(1.+pfact)
            T1 = T2*pfact
            T[i] = T1
            T[i+1] = T2
    #Upward pass
    for i in range(len(T)-2,-1,-1):
        T1,p1 = T[i],p[i]
        T2,p2 = T[i+1],p[i+1]
        pfact = (p1/p2)**Rcp
        if T1 < T2*pfact:
            Tbar = .5*(T1+T2) #Equal layer masses
                              #Not quite compatible with how
                              #heating is computed from flux
            T2 = 2.*Tbar/(1.+pfact)
            T1 = T2*pfact
            T[i] = T1
            T[i+1] = T2


#Define function to do time integration for n steps
def steps(Tg,T,p,q,nSteps,dtime):
    for i in range(nSteps):
        #Do smoothing
##        if i%20 == 0:
##            for j in range(1,len(T)-1):
##                T[j] = .25*T[j-1] + .5*T[j] + .25*T[j+1]
        flux,heat = radcomp(p,T,Tg,q)
        dT = heat*dtime
        #Limit the temperature change per step
        dT = numpy.where(dT>5.,5.,dT)
        dT = numpy.where(dT<-5.,-5.,dT)
        #Midpoint method time stepping
        #changed call to r.  Also modified to hold Tg fixed
        flux,heat = radcomp(p,T+.5*dT,Tg,q)
        dT = heat*dtime
        #Limit the temperature change per step
        dT = numpy.where(dT>5.,5.,dT)
        dT = numpy.where(dT<-5.,-5.,dT)
        T += dT
        #
        dTmax = max(abs(dT)) #To keep track of convergence

#   Do the surface balance
        kturb = .1
        T[-1] += -dtime*kturb*(T[-1] - Tg)
        #Dry adjustment step
        for iadj in range(10):
            dryAdj(T,p)
        Tad = T[-1]*(p/p[-1])**Rcp
        #** Temporary kludge to keep stratosphere from getting too cold
        T = numpy.where(T<50.,50.,T)  #**KLUDGE
        #
        #Dummies for separate LW and stellar. **FIX THIS**
        fluxStellar = fluxLW = heatStellar = heatLW = numpy.zeros(n)
    return Tg,Tad,T,flux,fluxStellar,fluxLW,heat,heatStellar,heatLW



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jan 23 12:16:20 2023

@authors: 
Tim Lichtenberg (TL)    
Ryan Boukrouche (RB)
"""

import numpy as np

def surf_Planck_nu(atm):
    h   = 6.63e-34
    c   = 3.0e8
    kb  = 1.38e-23
    B   = np.zeros(len(atm.band_centres))
    c1  = 1.191042e-5
    c2  = 1.4387752
    for i in range(len(atm.band_centres)):
        nu      = atm.band_centres[i]
        B[i]    = (c1*nu**3 / (np.exp(c2*nu/atm.ts)-1))
    B   = (1.-atm.albedo_s) * np.pi * B * atm.band_widths/1000.0
    return B


"""Import this file to set environment variables for socrates.

Based on:
https://github.com/nichollsh/SOCRATES/blob/main/sbin/set_rad_env_tmp
"""

from __future__ import annotations

from .socrates import SOCRATES_DIR

import os
import sys
import zipfile
from pathlib import Path
import logging

import click
import platformdirs
import requests

log = logging.getLogger("fwl."+__name__)

if not SOCRATES_DIR.exists():
    raise RuntimeError(f'Cannot find SOCRATES in this location: {SOCRATES_DIR}')

with open(SOCRATES_DIR / 'version') as f:
    SOCRATES_VERSION = f.readline()

log.debug('socrates location: %s' % SOCRATES_DIR)
log.debug('socrates version: %s' % SOCRATES_VERSION)

sep = os.pathsep

os.environ['RAD_DIR'] = str(SOCRATES_DIR)
os.environ['RAD_BIN'] = str(SOCRATES_DIR / 'bin')
os.environ['RAD_DATA'] = str(SOCRATES_DIR / 'data')
os.environ['RAD_SCRIPT'] = str(SOCRATES_DIR / 'sbin')
os.environ['LOCK_FILE'] = 'radiation_code.lock'
os.environ['PATH'] = str(SOCRATES_DIR / 'bin') + sep + os.environ['PATH']
os.environ['PATH'] = str(SOCRATES_DIR / 'sbin') + sep + os.environ['PATH']
os.environ['MANPATH'] = str(SOCRATES_DIR / 'man') + sep + os.environ.get('MANPATH', '')
sys.path.append(str(SOCRATES_DIR / 'python'))

os.environ['LD_LIBRARY_PATH'] = 'netcdfff' + sep + os.environ.get('LD_LIBRARY_PATH', '')
